      SUBROUTINE APGRID (UV,VIS,WT,L,G,CX,CY,N2,M,LROW,CNT,TY)
C-----------------------------------------------------------------------
C! FPS AP 120B load module
C# AP-appl
C-----------------------------------------------------------------------
C;  Copyright (C) 1995
C;  Associated Universities, Inc. Washington DC, USA.
C;
C;  This program is free software; you can redistribute it and/or
C;  modify it under the terms of the GNU General Public License as
C;  published by the Free Software Foundation; either version 2 of
C;  the License, or (at your option) any later version.
C;
C;  This program is distributed in the hope that it will be useful,
C;  but WITHOUT ANY WARRANTY; without even the implied warranty of
C;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C;  GNU General Public License for more details.
C;
C;  You should have received a copy of the GNU General Public
C;  License along with this program; if not, write to the Free
C;  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge,
C;  MA 02139, USA.
C;
C;  Correspondence concerning AIPS should be addressed as follows:
C;         Internet email: aipsmail@nrao.edu.
C;         Postal address: AIPS Project Office
C;                         National Radio Astronomy Observatory
C;                         520 Edgemont Road
C;                         Charlottesville, VA 22903-2475 USA
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C   ARRAY PROCESSOR MICROCODE FOR FPS 120-B ARRAY PROCESSOR
C
C
C     APGRID GRIDS VISIBILITY DATA THAT HAS BEEN LOADED INTO THE AP
C     PREVIOUSLY.  DATA IS FLOATED AND THEN SCALED; A TAPER IS APPLIED
C     TO THE WEIGHTS IF REQUESTED BEFORE GRIDDING.
C     WHEN TAPER IS REQUESTED AND TY=2 OR 3 LOCATIONS VIS-2 AND
C     VIS-1 ARE USED FOR WORK SPACE. IF TY=1 LOCATIONS VIS+2 AND
C     VIS+3 ARE USED.
C
C                UV = BASE ADDRESS OF U,V VECTOR
C                VIS = BASE ADDRESS OF VISIBILITY
C                WT = BASE ADDRESS OF WEIGHTS
C                L  = LENGTH OF VISIBILITY RECORD
C                G  = BASE ADDRESS OF GRID
C                CX = BASE ADDRESS OF ROW CONVOLVING FN. (Y ON SKY)
C                CY = BASE ADDRESS OF COL. CONVOLVING FN. (X ON SKY)
C                     CONVOLVING FNS TABULATED EVERY 1/100 CELL
C                N2 = INT ( (NO. CELLS USED ON A ROW)/2 )
C                M = NUMBER OF ROWS KEPT IN AP (MUST BE ODD)
C                LROW = LENGTH OF A ROW (V)
C                CNT =  NUMBER OF VISIBILITY POINTS.
C                TY  =  TYPE OF VISIBILITY DATA.
C                       1 = I MAPS
C                       2 = Q,U MAPS
C                       3 = V MAPS
C                 IF TY IS NEGATIVE THEN TAPERING IS REQUESTED.
C                 IF CNT IS NEG, DO NOT FLOAT OR SCALE.
C
C     EXPECTS NECESSARY CONSTANTS IN FOLLOWING AP LOCATIONS:
C
C           8 = -SIG(U)**2 (CELLS**2) FOR TAPER
C           9 = -SIG(V)**2 (CELLS**2) FOR TAPER
C          13 = UVSC SCALING FOR U,V
C          14 = SCVIS  SCALING FOR VISIBILITIES
C          15 = SCWT   SCALING FOR WEIGHTS
C
C  Programmer: W. D. Cotton.
C-----------------------------------------------------------------------
      INTEGER*2 CODE(2961),UV,VIS,WT,L,G,CX,CY,N2,M,LROW,CNT,
     *   TY,J1,J2,J3,J4,J5,J6,J7,J8,J9,J10,J11,J12,SLIST(16)
      COMMON /SPARY/ SLIST
      EQUIVALENCE (J1, SLIST( 1))
      EQUIVALENCE (J2, SLIST( 2))
      EQUIVALENCE (J3, SLIST( 3))
      EQUIVALENCE (J4, SLIST( 4))
      EQUIVALENCE (J5, SLIST( 5))
      EQUIVALENCE (J6, SLIST( 6))
      EQUIVALENCE (J7, SLIST( 7))
      EQUIVALENCE (J8, SLIST( 8))
      EQUIVALENCE (J9, SLIST( 9))
      EQUIVALENCE (J10, SLIST(10))
      EQUIVALENCE (J11, SLIST(11))
      EQUIVALENCE (J12, SLIST(12))
C
      DATA CODE(1) / 740/
      DATA CODE(    2),CODE(    3),CODE(    4),CODE(    5)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(    6),CODE(    7),CODE(    8),CODE(    9)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   10),CODE(   11),CODE(   12),CODE(   13)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   14),CODE(   15),CODE(   16),CODE(   17)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   18),CODE(   19),CODE(   20),CODE(   21)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   22),CODE(   23),CODE(   24),CODE(   25)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   26),CODE(   27),CODE(   28),CODE(   29)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   30),CODE(   31),CODE(   32),CODE(   33)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   34),CODE(   35),CODE(   36),CODE(   37)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   38),CODE(   39),CODE(   40),CODE(   41)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE(   42),CODE(   43),CODE(   44),CODE(   45)/
     *   Z120C,Z0000,Z0000,Z027C/
      DATA CODE(   46),CODE(   47),CODE(   48),CODE(   49)/
     *   Z0000,Z000C,Z0000,ZFFF5/
      DATA CODE(   50),CODE(   51),CODE(   52),CODE(   53)/
     *   Z120C,Z0000,Z0000,Z0297/
      DATA CODE(   54),CODE(   55),CODE(   56),CODE(   57)/
     *   Z0000,Z0001,Z0000,ZFFF8/
      DATA CODE(   58),CODE(   59),CODE(   60),CODE(   61)/
     *   Z120C,Z0000,Z0000,Z028A/
      DATA CODE(   62),CODE(   63),CODE(   64),CODE(   65)/
     *   Z0000,ZFFFA,Z0000,Z0000/
      DATA CODE(   66),CODE(   67),CODE(   68),CODE(   69)/
     *   Z120C,Z0000,Z0000,Z029E/
      DATA CODE(   70),CODE(   71),CODE(   72),CODE(   73)/
     *   Z0000,ZFFEF,Z0800,Z0001/
      DATA CODE(   74),CODE(   75),CODE(   76),CODE(   77)/
     *   Z120C,Z0000,Z0000,Z02B5/
      DATA CODE(   78),CODE(   79),CODE(   80),CODE(   81)/
     *   Z120C,Z0000,Z0000,Z0285/
      DATA CODE(   82),CODE(   83),CODE(   84),CODE(   85)/
     *   Z0000,ZFFF2,Z0000,Z0000/
      DATA CODE(   86),CODE(   87),CODE(   88),CODE(   89)/
     *   Z120C,Z0000,Z0000,Z0299/
      DATA CODE(   90),CODE(   91),CODE(   92),CODE(   93)/
     *   Z0000,ZFFEF,Z0800,Z0000/
      DATA CODE(   94),CODE(   95),CODE(   96),CODE(   97)/
     *   Z120C,Z0000,Z0000,Z02B1/
      DATA CODE(   98),CODE(   99),CODE(  100),CODE(  101)/
     *   Z1004,Z0192,Z0000,Z0000/
      DATA CODE(  102),CODE(  103),CODE(  104),CODE(  105)/
     *   Z1208,Z0000,Z0000,Z000C/
      DATA CODE(  106),CODE(  107),CODE(  108),CODE(  109)/
     *   Z120C,Z0000,Z0000,Z0294/
      DATA CODE(  110),CODE(  111),CODE(  112),CODE(  113)/
     *   Z0000,ZFFEA,Z0800,Z0000/
      DATA CODE(  114),CODE(  115),CODE(  116),CODE(  117)/
     *   Z120C,Z0000,Z0000,Z02AC/
      DATA CODE(  118),CODE(  119),CODE(  120),CODE(  121)/
     *   Z0000,Z01B2,Z0000,Z0000/
      DATA CODE(  122),CODE(  123),CODE(  124),CODE(  125)/
     *   Z1208,Z0000,Z0000,Z011D/
      DATA CODE(  126),CODE(  127),CODE(  128),CODE(  129)/
     *   Z120C,Z0000,Z0000,Z0284/
      DATA CODE(  130),CODE(  131),CODE(  132),CODE(  133)/
     *   Z0000,Z0001,Z0000,ZFFE5/
      DATA CODE(  134),CODE(  135),CODE(  136),CODE(  137)/
     *   Z120C,Z0000,Z0000,Z02A4/
      DATA CODE(  138),CODE(  139),CODE(  140),CODE(  141)/
     *   Z120C,Z0000,Z0000,Z0276/
      DATA CODE(  142),CODE(  143),CODE(  144),CODE(  145)/
     *   Z0000,ZFFE2,Z0000,Z0000/
      DATA CODE(  146),CODE(  147),CODE(  148),CODE(  149)/
     *   Z1208,Z0000,Z0000,Z0047/
      DATA CODE(  150),CODE(  151),CODE(  152),CODE(  153)/
     *   Z120C,Z0000,Z0000,Z027E/
      DATA CODE(  154),CODE(  155),CODE(  156),CODE(  157)/
     *   Z0000,Z0005,Z0000,ZFFDA/
      DATA CODE(  158),CODE(  159),CODE(  160),CODE(  161)/
     *   Z0400,ZFFDA,Z0000,ZFFD9/
      DATA CODE(  162),CODE(  163),CODE(  164),CODE(  165)/
     *   Z0400,ZFFD9,Z0000,ZFFDD/
      DATA CODE(  166),CODE(  167),CODE(  168),CODE(  169)/
     *   Z120C,Z0000,Z0000,Z0251/
      DATA CODE(  170),CODE(  171),CODE(  172),CODE(  173)/
     *   Z120C,Z0000,Z0000,Z0243/
      DATA CODE(  174),CODE(  175),CODE(  176),CODE(  177)/
     *   Z120C,Z0000,Z0000,Z0278/
      DATA CODE(  178),CODE(  179),CODE(  180),CODE(  181)/
     *   Z0000,Z0005,Z0000,ZFFDA/
      DATA CODE(  182),CODE(  183),CODE(  184),CODE(  185)/
     *   Z0400,ZFFD4,Z0000,ZFFD9/
      DATA CODE(  186),CODE(  187),CODE(  188),CODE(  189)/
     *   Z0400,ZFFD3,Z0000,ZFFD7/
      DATA CODE(  190),CODE(  191),CODE(  192),CODE(  193)/
     *   Z120C,Z0000,Z0000,Z024B/
      DATA CODE(  194),CODE(  195),CODE(  196),CODE(  197)/
     *   Z120C,Z0000,Z0000,Z023D/
      DATA CODE(  198),CODE(  199),CODE(  200),CODE(  201)/
     *   Z120C,Z0000,Z0000,Z0272/
      DATA CODE(  202),CODE(  203),CODE(  204),CODE(  205)/
     *   Z0000,Z0005,Z0400,ZFFCE/
      DATA CODE(  206),CODE(  207),CODE(  208),CODE(  209)/
     *   Z0400,ZFFCE,Z0400,ZFFCD/
      DATA CODE(  210),CODE(  211),CODE(  212),CODE(  213)/
     *   Z0400,ZFFCD,Z0000,ZFFD1/
      DATA CODE(  214),CODE(  215),CODE(  216),CODE(  217)/
     *   Z120C,Z0000,Z0000,Z0245/
      DATA CODE(  218),CODE(  219),CODE(  220),CODE(  221)/
     *   Z120C,Z0000,Z0000,Z0237/
      DATA CODE(  222),CODE(  223),CODE(  224),CODE(  225)/
     *   Z120C,Z0000,Z0000,Z0277/
      DATA CODE(  226),CODE(  227),CODE(  228),CODE(  229)/
     *   Z0400,ZFFC8,Z0800,Z0001/
      DATA CODE(  230),CODE(  231),CODE(  232),CODE(  233)/
     *   Z120C,Z0000,Z0000,Z028E/
      DATA CODE(  234),CODE(  235),CODE(  236),CODE(  237)/
     *   Z120C,Z0000,Z0000,Z025E/
      DATA CODE(  238),CODE(  239),CODE(  240),CODE(  241)/
     *   Z0400,ZFFCB,Z0000,Z0000/
      DATA CODE(  242),CODE(  243),CODE(  244),CODE(  245)/
     *   Z120C,Z0000,Z0000,Z0267/
      DATA CODE(  246),CODE(  247),CODE(  248),CODE(  249)/
     *   Z0000,Z0005,Z0400,ZFFC9/
      DATA CODE(  250),CODE(  251),CODE(  252),CODE(  253)/
     *   Z0400,ZFFC3,Z0400,ZFFC8/
      DATA CODE(  254),CODE(  255),CODE(  256),CODE(  257)/
     *   Z0400,ZFFC2,Z0000,ZFFC6/
      DATA CODE(  258),CODE(  259),CODE(  260),CODE(  261)/
     *   Z120C,Z0000,Z0000,Z023A/
      DATA CODE(  262),CODE(  263),CODE(  264),CODE(  265)/
     *   Z120C,Z0000,Z0000,Z022C/
      DATA CODE(  266),CODE(  267),CODE(  268),CODE(  269)/
     *   Z120C,Z0000,Z0000,Z0261/
      DATA CODE(  270),CODE(  271),CODE(  272),CODE(  273)/
     *   Z0000,Z0005,Z0000,ZFFBE/
      DATA CODE(  274),CODE(  275),CODE(  276),CODE(  277)/
     *   Z0400,ZFFBD,Z0000,ZFFBD/
      DATA CODE(  278),CODE(  279),CODE(  280),CODE(  281)/
     *   Z0400,ZFFBC,Z0000,ZFFC0/
      DATA CODE(  282),CODE(  283),CODE(  284),CODE(  285)/
     *   Z120C,Z0000,Z0000,Z0234/
      DATA CODE(  286),CODE(  287),CODE(  288),CODE(  289)/
     *   Z120C,Z0000,Z0000,Z0226/
      DATA CODE(  290),CODE(  291),CODE(  292),CODE(  293)/
     *   Z120C,Z0000,Z0000,Z025B/
      DATA CODE(  294),CODE(  295),CODE(  296),CODE(  297)/
     *   Z0000,Z0006,Z0000,ZFFB7/
      DATA CODE(  298),CODE(  299),CODE(  300),CODE(  301)/
     *   Z0400,ZFFB7,Z0800,Z000D/
      DATA CODE(  302),CODE(  303),CODE(  304),CODE(  305)/
     *   Z0000,ZFFB5,Z0400,ZFFB6/
      DATA CODE(  306),CODE(  307),CODE(  308),CODE(  309)/
     *   Z0000,ZFFB9,Z0000,Z0000/
      DATA CODE(  310),CODE(  311),CODE(  312),CODE(  313)/
     *   Z120C,Z0000,Z0000,Z022D/
      DATA CODE(  314),CODE(  315),CODE(  316),CODE(  317)/
     *   Z120C,Z0000,Z0000,Z01CC/
      DATA CODE(  318),CODE(  319),CODE(  320),CODE(  321)/
     *   Z120C,Z0000,Z0000,Z0254/
      DATA CODE(  322),CODE(  323),CODE(  324),CODE(  325)/
     *   Z0000,Z0006,Z0000,ZFFB6/
      DATA CODE(  326),CODE(  327),CODE(  328),CODE(  329)/
     *   Z0400,ZFFB0,Z0800,Z000D/
      DATA CODE(  330),CODE(  331),CODE(  332),CODE(  333)/
     *   Z0000,ZFFB4,Z0400,ZFFAF/
      DATA CODE(  334),CODE(  335),CODE(  336),CODE(  337)/
     *   Z0000,ZFFB2,Z0000,Z0000/
      DATA CODE(  338),CODE(  339),CODE(  340),CODE(  341)/
     *   Z120C,Z0000,Z0000,Z0226/
      DATA CODE(  342),CODE(  343),CODE(  344),CODE(  345)/
     *   Z120C,Z0000,Z0000,Z01C5/
      DATA CODE(  346),CODE(  347),CODE(  348),CODE(  349)/
     *   Z120C,Z0000,Z0000,Z024D/
      DATA CODE(  350),CODE(  351),CODE(  352),CODE(  353)/
     *   Z0000,Z0006,Z0400,ZFFA9/
      DATA CODE(  354),CODE(  355),CODE(  356),CODE(  357)/
     *   Z0400,ZFFA9,Z0800,Z000E/
      DATA CODE(  358),CODE(  359),CODE(  360),CODE(  361)/
     *   Z0400,ZFFA7,Z0400,ZFFA8/
      DATA CODE(  362),CODE(  363),CODE(  364),CODE(  365)/
     *   Z0000,ZFFAB,Z0000,Z0000/
      DATA CODE(  366),CODE(  367),CODE(  368),CODE(  369)/
     *   Z120C,Z0000,Z0000,Z021F/
      DATA CODE(  370),CODE(  371),CODE(  372),CODE(  373)/
     *   Z120C,Z0000,Z0000,Z01BE/
      DATA CODE(  374),CODE(  375),CODE(  376),CODE(  377)/
     *   Z120C,Z0000,Z0000,Z0246/
      DATA CODE(  378),CODE(  379),CODE(  380),CODE(  381)/
     *   Z0000,Z0006,Z0400,ZFFA8/
      DATA CODE(  382),CODE(  383),CODE(  384),CODE(  385)/
     *   Z0400,ZFFA2,Z0800,Z000E/
      DATA CODE(  386),CODE(  387),CODE(  388),CODE(  389)/
     *   Z0400,ZFFA6,Z0400,ZFFA1/
      DATA CODE(  390),CODE(  391),CODE(  392),CODE(  393)/
     *   Z0000,ZFFA4,Z0000,Z0000/
      DATA CODE(  394),CODE(  395),CODE(  396),CODE(  397)/
     *   Z120C,Z0000,Z0000,Z0218/
      DATA CODE(  398),CODE(  399),CODE(  400),CODE(  401)/
     *   Z120C,Z0000,Z0000,Z01B7/
      DATA CODE(  402),CODE(  403),CODE(  404),CODE(  405)/
     *   Z120C,Z0000,Z0000,Z023F/
      DATA CODE(  406),CODE(  407),CODE(  408),CODE(  409)/
     *   Z0000,Z0006,Z0000,ZFF9C/
      DATA CODE(  410),CODE(  411),CODE(  412),CODE(  413)/
     *   Z0400,ZFF9B,Z0800,Z000F/
      DATA CODE(  414),CODE(  415),CODE(  416),CODE(  417)/
     *   Z0000,ZFF9A,Z0400,ZFF9A/
      DATA CODE(  418),CODE(  419),CODE(  420),CODE(  421)/
     *   Z0000,ZFF9D,Z0000,Z0000/
      DATA CODE(  422),CODE(  423),CODE(  424),CODE(  425)/
     *   Z120C,Z0000,Z0000,Z0211/
      DATA CODE(  426),CODE(  427),CODE(  428),CODE(  429)/
     *   Z120C,Z0000,Z0000,Z01B0/
      DATA CODE(  430),CODE(  431),CODE(  432),CODE(  433)/
     *   Z120C,Z0000,Z0000,Z0243/
      DATA CODE(  434),CODE(  435),CODE(  436),CODE(  437)/
     *   Z0400,ZFF99,Z0800,Z0000/
      DATA CODE(  438),CODE(  439),CODE(  440),CODE(  441)/
     *   Z120C,Z0000,Z0000,Z025B/
      DATA CODE(  442),CODE(  443),CODE(  444),CODE(  445)/
     *   Z1004,Z0192,Z0000,Z0000/
      DATA CODE(  446),CODE(  447),CODE(  448),CODE(  449)/
     *   Z1208,Z0000,Z0000,Z004E/
      DATA CODE(  450),CODE(  451),CODE(  452),CODE(  453)/
     *   Z120C,Z0000,Z0000,Z0233/
      DATA CODE(  454),CODE(  455),CODE(  456),CODE(  457)/
     *   Z0000,Z0001,Z0400,ZFF94/
      DATA CODE(  458),CODE(  459),CODE(  460),CODE(  461)/
     *   Z120C,Z0000,Z0000,Z0253/
      DATA CODE(  462),CODE(  463),CODE(  464),CODE(  465)/
     *   Z120C,Z0000,Z0000,Z0225/
      DATA CODE(  466),CODE(  467),CODE(  468),CODE(  469)/
     *   Z0400,ZFF91,Z0000,Z0000/
      DATA CODE(  470),CODE(  471),CODE(  472),CODE(  473)/
     *   Z120C,Z0000,Z0000,Z0239/
      DATA CODE(  474),CODE(  475),CODE(  476),CODE(  477)/
     *   Z0400,ZFF8F,Z0800,Z0001/
      DATA CODE(  478),CODE(  479),CODE(  480),CODE(  481)/
     *   Z120C,Z0000,Z0000,Z0251/
      DATA CODE(  482),CODE(  483),CODE(  484),CODE(  485)/
     *   Z0000,Z01B2,Z0000,Z0000/
      DATA CODE(  486),CODE(  487),CODE(  488),CODE(  489)/
     *   Z1208,Z0000,Z0000,Z000C/
      DATA CODE(  490),CODE(  491),CODE(  492),CODE(  493)/
     *   Z120C,Z0000,Z0000,Z0234/
      DATA CODE(  494),CODE(  495),CODE(  496),CODE(  497)/
     *   Z0400,ZFF85,Z0800,Z0002/
      DATA CODE(  498),CODE(  499),CODE(  500),CODE(  501)/
     *   Z120C,Z0000,Z0000,Z024C/
      DATA CODE(  502),CODE(  503),CODE(  504),CODE(  505)/
     *   Z120C,Z0000,Z0000,Z021B/
      DATA CODE(  506),CODE(  507),CODE(  508),CODE(  509)/
     *   Z0000,ZFF89,Z0000,Z0000/
      DATA CODE(  510),CODE(  511),CODE(  512),CODE(  513)/
     *   Z120C,Z0000,Z0000,Z022F/
      DATA CODE(  514),CODE(  515),CODE(  516),CODE(  517)/
     *   Z0400,ZFF80,Z0800,Z0001/
      DATA CODE(  518),CODE(  519),CODE(  520),CODE(  521)/
     *   Z120C,Z0000,Z0000,Z0247/
      DATA CODE(  522),CODE(  523),CODE(  524),CODE(  525)/
     *   Z120C,Z0000,Z0000,Z0216/
      DATA CODE(  526),CODE(  527),CODE(  528),CODE(  529)/
     *   Z0400,ZFF84,Z0000,Z0000/
      DATA CODE(  530),CODE(  531),CODE(  532),CODE(  533)/
     *   Z1208,Z0000,Z0000,Z000B/
      DATA CODE(  534),CODE(  535),CODE(  536),CODE(  537)/
     *   Z120C,Z0000,Z0000,Z0229/
      DATA CODE(  538),CODE(  539),CODE(  540),CODE(  541)/
     *   Z0400,ZFF7A,Z0800,Z0002/
      DATA CODE(  542),CODE(  543),CODE(  544),CODE(  545)/
     *   Z120C,Z0000,Z0000,Z0240/
      DATA CODE(  546),CODE(  547),CODE(  548),CODE(  549)/
     *   Z120C,Z0000,Z0000,Z0210/
      DATA CODE(  550),CODE(  551),CODE(  552),CODE(  553)/
     *   Z0000,ZFF7E,Z0000,Z0000/
      DATA CODE(  554),CODE(  555),CODE(  556),CODE(  557)/
     *   Z120C,Z0000,Z0000,Z0224/
      DATA CODE(  558),CODE(  559),CODE(  560),CODE(  561)/
     *   Z0400,ZFF75,Z0800,Z0003/
      DATA CODE(  562),CODE(  563),CODE(  564),CODE(  565)/
     *   Z120C,Z0000,Z0000,Z023B/
      DATA CODE(  566),CODE(  567),CODE(  568),CODE(  569)/
     *   Z120C,Z0000,Z0000,Z020B/
      DATA CODE(  570),CODE(  571),CODE(  572),CODE(  573)/
     *   Z0400,ZFF79,Z0000,Z0000/
      DATA CODE(  574),CODE(  575),CODE(  576),CODE(  577)/
     *   Z120C,Z0000,Z0000,Z0214/
      DATA CODE(  578),CODE(  579),CODE(  580),CODE(  581)/
     *   Z0000,Z0005,Z0000,ZFF70/
      DATA CODE(  582),CODE(  583),CODE(  584),CODE(  585)/
     *   Z0400,ZFF70,Z0000,ZFF76/
      DATA CODE(  586),CODE(  587),CODE(  588),CODE(  589)/
     *   Z0400,ZFF6F,Z0000,ZFF73/
      DATA CODE(  590),CODE(  591),CODE(  592),CODE(  593)/
     *   Z120C,Z0000,Z0000,Z01E7/
      DATA CODE(  594),CODE(  595),CODE(  596),CODE(  597)/
     *   Z120C,Z0000,Z0000,Z0194/
      DATA CODE(  598),CODE(  599),CODE(  600),CODE(  601)/
     *   Z120C,Z0000,Z0000,Z020E/
      DATA CODE(  602),CODE(  603),CODE(  604),CODE(  605)/
     *   Z0000,Z0005,Z0000,ZFF70/
      DATA CODE(  606),CODE(  607),CODE(  608),CODE(  609)/
     *   Z0400,ZFF6A,Z0400,ZFF70/
      DATA CODE(  610),CODE(  611),CODE(  612),CODE(  613)/
     *   Z0400,ZFF69,Z0000,ZFF6D/
      DATA CODE(  614),CODE(  615),CODE(  616),CODE(  617)/
     *   Z120C,Z0000,Z0000,Z01E1/
      DATA CODE(  618),CODE(  619),CODE(  620),CODE(  621)/
     *   Z120C,Z0000,Z0000,Z018E/
      DATA CODE(  622),CODE(  623),CODE(  624),CODE(  625)/
     *   Z120C,Z0000,Z0000,Z0208/
      DATA CODE(  626),CODE(  627),CODE(  628),CODE(  629)/
     *   Z0000,Z0006,Z0000,ZFF6B/
      DATA CODE(  630),CODE(  631),CODE(  632),CODE(  633)/
     *   Z0400,ZFF64,Z0800,Z0008/
      DATA CODE(  634),CODE(  635),CODE(  636),CODE(  637)/
     *   Z0000,ZFF69,Z0400,ZFF63/
      DATA CODE(  638),CODE(  639),CODE(  640),CODE(  641)/
     *   Z0000,ZFF66,Z0000,Z0000/
      DATA CODE(  642),CODE(  643),CODE(  644),CODE(  645)/
     *   Z120C,Z0000,Z0000,Z01DA/
      DATA CODE(  646),CODE(  647),CODE(  648),CODE(  649)/
     *   Z120C,Z0000,Z0000,Z0179/
      DATA CODE(  650),CODE(  651),CODE(  652),CODE(  653)/
     *   Z120C,Z0000,Z0000,Z0201/
      DATA CODE(  654),CODE(  655),CODE(  656),CODE(  657)/
     *   Z0000,Z0006,Z0400,ZFF64/
      DATA CODE(  658),CODE(  659),CODE(  660),CODE(  661)/
     *   Z0400,ZFF5D,Z0800,Z0009/
      DATA CODE(  662),CODE(  663),CODE(  664),CODE(  665)/
     *   Z0400,ZFF62,Z0400,ZFF5C/
      DATA CODE(  666),CODE(  667),CODE(  668),CODE(  669)/
     *   Z0000,ZFF5F,Z0000,Z0000/
      DATA CODE(  670),CODE(  671),CODE(  672),CODE(  673)/
     *   Z120C,Z0000,Z0000,Z01D3/
      DATA CODE(  674),CODE(  675),CODE(  676),CODE(  677)/
     *   Z120C,Z0000,Z0000,Z0172/
      DATA CODE(  678),CODE(  679),CODE(  680),CODE(  681)/
     *   Z120C,Z0000,Z0000,Z01FA/
      DATA CODE(  682),CODE(  683),CODE(  684),CODE(  685)/
     *   Z0000,Z0007,Z0000,ZFF5D/
      DATA CODE(  686),CODE(  687),CODE(  688),CODE(  689)/
     *   Z0400,ZFF56,Z0400,ZFF5C/
      DATA CODE(  690),CODE(  691),CODE(  692),CODE(  693)/
     *   Z0400,ZFF55,Z0000,ZFF5B/
      DATA CODE(  694),CODE(  695),CODE(  696),CODE(  697)/
     *   Z0400,ZFF54,Z0000,ZFF58/
      DATA CODE(  698),CODE(  699),CODE(  700),CODE(  701)/
     *   Z120C,Z0000,Z0000,Z01CC/
      DATA CODE(  702),CODE(  703),CODE(  704),CODE(  705)/
     *   Z120C,Z0000,Z0000,Z014F/
      DATA CODE(  706),CODE(  707),CODE(  708),CODE(  709)/
     *   Z120C,Z0000,Z0000,Z01F3/
      DATA CODE(  710),CODE(  711),CODE(  712),CODE(  713)/
     *   Z0000,Z0005,Z0000,ZFF56/
      DATA CODE(  714),CODE(  715),CODE(  716),CODE(  717)/
     *   Z0400,ZFF4F,Z0000,ZFF55/
      DATA CODE(  718),CODE(  719),CODE(  720),CODE(  721)/
     *   Z0400,ZFF4E,Z0000,ZFF52/
      DATA CODE(  722),CODE(  723),CODE(  724),CODE(  725)/
     *   Z120C,Z0000,Z0000,Z01C6/
      DATA CODE(  726),CODE(  727),CODE(  728),CODE(  729)/
     *   Z120C,Z0000,Z0000,Z017E/
      DATA CODE(  730),CODE(  731),CODE(  732),CODE(  733)/
     *   Z120C,Z0000,Z0000,Z01ED/
      DATA CODE(  734),CODE(  735),CODE(  736),CODE(  737)/
     *   Z0000,Z0007,Z0000,ZFF4A/
      DATA CODE(  738),CODE(  739),CODE(  740),CODE(  741)/
     *   Z0400,ZFF49,Z0000,ZFF4F/
      DATA CODE(  742),CODE(  743),CODE(  744),CODE(  745)/
     *   Z0400,ZFF48,Z0000,ZFF48/
      DATA CODE(  746),CODE(  747),CODE(  748),CODE(  749)/
     *   Z0400,ZFF47,Z0000,ZFF4B/
      DATA CODE(  750),CODE(  751),CODE(  752),CODE(  753)/
     *   Z120C,Z0000,Z0000,Z01BF/
      DATA CODE(  754),CODE(  755),CODE(  756),CODE(  757)/
     *   Z120C,Z0000,Z0000,Z0150/
      DATA CODE(  758),CODE(  759),CODE(  760),CODE(  761)/
     *   Z120C,Z0000,Z0000,Z01F1/
      DATA CODE(  762),CODE(  763),CODE(  764),CODE(  765)/
     *   Z0400,ZFF47,Z0800,Z0002/
      DATA CODE(  766),CODE(  767),CODE(  768),CODE(  769)/
     *   Z120C,Z0000,Z0000,Z0209/
      DATA CODE(  770),CODE(  771),CODE(  772),CODE(  773)/
     *   Z0000,Z01B2,Z0000,Z0000/
      DATA CODE(  774),CODE(  775),CODE(  776),CODE(  777)/
     *   Z1208,Z0000,Z0000,Z0029/
      DATA CODE(  778),CODE(  779),CODE(  780),CODE(  781)/
     *   Z120C,Z0000,Z0000,Z01E1/
      DATA CODE(  782),CODE(  783),CODE(  784),CODE(  785)/
     *   Z0000,Z0009,Z0000,ZFF3D/
      DATA CODE(  786),CODE(  787),CODE(  788),CODE(  789)/
     *   Z0400,ZFF3C,Z0000,ZFF3D/
      DATA CODE(  790),CODE(  791),CODE(  792),CODE(  793)/
     *   Z0000,ZFF3D,Z0400,ZFF3D/
      DATA CODE(  794),CODE(  795),CODE(  796),CODE(  797)/
     *   Z0000,ZFF3D,Z0400,ZFF3D/
      DATA CODE(  798),CODE(  799),CODE(  800),CODE(  801)/
     *   Z0000,ZFF3D,Z0400,ZFF3D/
      DATA CODE(  802),CODE(  803),CODE(  804),CODE(  805)/
     *   Z120C,Z0000,Z0000,Z01B2/
      DATA CODE(  806),CODE(  807),CODE(  808),CODE(  809)/
     *   Z120C,Z0000,Z0000,Z0073/
      DATA CODE(  810),CODE(  811),CODE(  812),CODE(  813)/
     *   Z120C,Z0000,Z0000,Z01D9/
      DATA CODE(  814),CODE(  815),CODE(  816),CODE(  817)/
     *   Z0000,Z0001,Z0400,ZFF36/
      DATA CODE(  818),CODE(  819),CODE(  820),CODE(  821)/
     *   Z4028,Z0000,Z0000,Z0000/
      DATA CODE(  822),CODE(  823),CODE(  824),CODE(  825)/
     *   Z4284,Z0000,Z0000,Z0000/
      DATA CODE(  826),CODE(  827),CODE(  828),CODE(  829)/
     *   Z120C,Z0000,Z0000,Z01D5/
      DATA CODE(  830),CODE(  831),CODE(  832),CODE(  833)/
     *   Z0000,Z0001,Z0000,ZFF31/
      DATA CODE(  834),CODE(  835),CODE(  836),CODE(  837)/
     *   Z120C,Z0000,Z0000,Z01F7/
      DATA CODE(  838),CODE(  839),CODE(  840),CODE(  841)/
     *   Z120C,Z0000,Z0000,Z01C7/
      DATA CODE(  842),CODE(  843),CODE(  844),CODE(  845)/
     *   Z0000,ZFF2E,Z0000,Z0000/
      DATA CODE(  846),CODE(  847),CODE(  848),CODE(  849)/
     *   Z4284,Z0000,Z0000,Z0000/
      DATA CODE(  850),CODE(  851),CODE(  852),CODE(  853)/
     *   Z120C,Z0000,Z0000,Z01CF/
      DATA CODE(  854),CODE(  855),CODE(  856),CODE(  857)/
     *   Z0000,Z0001,Z0400,ZFF2B/
      DATA CODE(  858),CODE(  859),CODE(  860),CODE(  861)/
     *   Z120C,Z0000,Z0000,Z01F1/
      DATA CODE(  862),CODE(  863),CODE(  864),CODE(  865)/
     *   Z120C,Z0000,Z0000,Z01C1/
      DATA CODE(  866),CODE(  867),CODE(  868),CODE(  869)/
     *   Z0400,ZFF28,Z0000,Z0000/
      DATA CODE(  870),CODE(  871),CODE(  872),CODE(  873)/
     *   Z4284,Z0000,Z0000,Z0000/
      DATA CODE(  874),CODE(  875),CODE(  876),CODE(  877)/
     *   Z120C,Z0000,Z0000,Z01C9/
      DATA CODE(  878),CODE(  879),CODE(  880),CODE(  881)/
     *   Z0000,Z0001,Z0000,ZFF26/
      DATA CODE(  882),CODE(  883),CODE(  884),CODE(  885)/
     *   Z120C,Z0000,Z0000,Z01EB/
      DATA CODE(  886),CODE(  887),CODE(  888),CODE(  889)/
     *   Z120C,Z0000,Z0000,Z01BB/
      DATA CODE(  890),CODE(  891),CODE(  892),CODE(  893)/
     *   Z0000,ZFF23,Z0000,Z0000/
      DATA CODE(  894),CODE(  895),CODE(  896),CODE(  897)/
     *   Z120C,Z0000,Z0000,Z01CF/
      DATA CODE(  898),CODE(  899),CODE(  900),CODE(  901)/
     *   Z0000,ZFF25,Z0800,Z0001/
      DATA CODE(  902),CODE(  903),CODE(  904),CODE(  905)/
     *   Z120C,Z0000,Z0000,Z01E7/
      DATA CODE(  906),CODE(  907),CODE(  908),CODE(  909)/
     *   Z120C,Z0000,Z0000,Z01B6/
      DATA CODE(  910),CODE(  911),CODE(  912),CODE(  913)/
     *   Z0000,ZFF22,Z0000,Z0000/
      DATA CODE(  914),CODE(  915),CODE(  916),CODE(  917)/
     *   Z120C,Z0000,Z0000,Z01CA/
      DATA CODE(  918),CODE(  919),CODE(  920),CODE(  921)/
     *   Z0000,ZFF20,Z0800,Z0000/
      DATA CODE(  922),CODE(  923),CODE(  924),CODE(  925)/
     *   Z120C,Z0000,Z0000,Z01E2/
      DATA CODE(  926),CODE(  927),CODE(  928),CODE(  929)/
     *   Z1004,Z0192,Z0000,Z0000/
      DATA CODE(  930),CODE(  931),CODE(  932),CODE(  933)/
     *   Z1208,Z0000,Z0000,ZFFDA/
      DATA CODE(  934),CODE(  935),CODE(  936),CODE(  937)/
     *   Z1208,Z0000,Z0000,Z0052/
      DATA CODE(  938),CODE(  939),CODE(  940),CODE(  941)/
     *   Z120C,Z0000,Z0000,Z01C4/
      DATA CODE(  942),CODE(  943),CODE(  944),CODE(  945)/
     *   Z0000,ZFF1E,Z0800,Z0000/
      DATA CODE(  946),CODE(  947),CODE(  948),CODE(  949)/
     *   Z120C,Z0000,Z0000,Z01DC/
      DATA CODE(  950),CODE(  951),CODE(  952),CODE(  953)/
     *   Z0000,Z01D2,Z0000,Z0000/
      DATA CODE(  954),CODE(  955),CODE(  956),CODE(  957)/
     *   Z1208,Z0000,Z0000,Z0025/
      DATA CODE(  958),CODE(  959),CODE(  960),CODE(  961)/
     *   Z120C,Z0000,Z0000,Z01BF/
      DATA CODE(  962),CODE(  963),CODE(  964),CODE(  965)/
     *   Z0400,ZFF10,Z0800,Z0002/
      DATA CODE(  966),CODE(  967),CODE(  968),CODE(  969)/
     *   Z120C,Z0000,Z0000,Z01D6/
      DATA CODE(  970),CODE(  971),CODE(  972),CODE(  973)/
     *   Z120C,Z0000,Z0000,Z01A6/
      DATA CODE(  974),CODE(  975),CODE(  976),CODE(  977)/
     *   Z0000,ZFF15,Z0000,Z0000/
      DATA CODE(  978),CODE(  979),CODE(  980),CODE(  981)/
     *   Z120C,Z0000,Z0000,Z01BA/
      DATA CODE(  982),CODE(  983),CODE(  984),CODE(  985)/
     *   Z0400,ZFF0B,Z0800,Z0003/
      DATA CODE(  986),CODE(  987),CODE(  988),CODE(  989)/
     *   Z120C,Z0000,Z0000,Z01D1/
      DATA CODE(  990),CODE(  991),CODE(  992),CODE(  993)/
     *   Z120C,Z0000,Z0000,Z01A1/
      DATA CODE(  994),CODE(  995),CODE(  996),CODE(  997)/
     *   Z0400,ZFF10,Z0000,Z0000/
      DATA CODE(  998),CODE(  999),CODE( 1000),CODE( 1001)/
     *   Z120C,Z0000,Z0000,Z01AA/
      DATA CODE( 1002),CODE( 1003),CODE( 1004),CODE( 1005)/
     *   Z0000,Z0005,Z0000,ZFF0E/
      DATA CODE( 1006),CODE( 1007),CODE( 1008),CODE( 1009)/
     *   Z0400,ZFF06,Z0000,ZFF0D/
      DATA CODE( 1010),CODE( 1011),CODE( 1012),CODE( 1013)/
     *   Z0400,ZFF05,Z0000,ZFF09/
      DATA CODE( 1014),CODE( 1015),CODE( 1016),CODE( 1017)/
     *   Z120C,Z0000,Z0000,Z017D/
      DATA CODE( 1018),CODE( 1019),CODE( 1020),CODE( 1021)/
     *   Z120C,Z0000,Z0000,Z016F/
      DATA CODE( 1022),CODE( 1023),CODE( 1024),CODE( 1025)/
     *   Z120C,Z0000,Z0000,Z01A4/
      DATA CODE( 1026),CODE( 1027),CODE( 1028),CODE( 1029)/
     *   Z0000,Z0005,Z0400,ZFF08/
      DATA CODE( 1030),CODE( 1031),CODE( 1032),CODE( 1033)/
     *   Z0400,ZFF00,Z0400,ZFF07/
      DATA CODE( 1034),CODE( 1035),CODE( 1036),CODE( 1037)/
     *   Z0400,ZFEFF,Z0000,ZFF03/
      DATA CODE( 1038),CODE( 1039),CODE( 1040),CODE( 1041)/
     *   Z120C,Z0000,Z0000,Z0177/
      DATA CODE( 1042),CODE( 1043),CODE( 1044),CODE( 1045)/
     *   Z120C,Z0000,Z0000,Z0169/
      DATA CODE( 1046),CODE( 1047),CODE( 1048),CODE( 1049)/
     *   Z120C,Z0000,Z0000,Z019E/
      DATA CODE( 1050),CODE( 1051),CODE( 1052),CODE( 1053)/
     *   Z0000,Z0006,Z0000,ZFF02/
      DATA CODE( 1054),CODE( 1055),CODE( 1056),CODE( 1057)/
     *   Z0400,ZFEFA,Z0800,Z000E/
      DATA CODE( 1058),CODE( 1059),CODE( 1060),CODE( 1061)/
     *   Z0000,ZFF00,Z0400,ZFEF9/
      DATA CODE( 1062),CODE( 1063),CODE( 1064),CODE( 1065)/
     *   Z0000,ZFEFC,Z0000,Z0000/
      DATA CODE( 1066),CODE( 1067),CODE( 1068),CODE( 1069)/
     *   Z120C,Z0000,Z0000,Z0170/
      DATA CODE( 1070),CODE( 1071),CODE( 1072),CODE( 1073)/
     *   Z120C,Z0000,Z0000,Z010F/
      DATA CODE( 1074),CODE( 1075),CODE( 1076),CODE( 1077)/
     *   Z120C,Z0000,Z0000,Z0197/
      DATA CODE( 1078),CODE( 1079),CODE( 1080),CODE( 1081)/
     *   Z0000,Z0006,Z0400,ZFEFB/
      DATA CODE( 1082),CODE( 1083),CODE( 1084),CODE( 1085)/
     *   Z0400,ZFEF3,Z0800,Z000E/
      DATA CODE( 1086),CODE( 1087),CODE( 1088),CODE( 1089)/
     *   Z0400,ZFEF9,Z0400,ZFEF2/
      DATA CODE( 1090),CODE( 1091),CODE( 1092),CODE( 1093)/
     *   Z0000,ZFEF5,Z0000,Z0000/
      DATA CODE( 1094),CODE( 1095),CODE( 1096),CODE( 1097)/
     *   Z120C,Z0000,Z0000,Z0169/
      DATA CODE( 1098),CODE( 1099),CODE( 1100),CODE( 1101)/
     *   Z120C,Z0000,Z0000,Z0108/
      DATA CODE( 1102),CODE( 1103),CODE( 1104),CODE( 1105)/
     *   Z120C,Z0000,Z0000,Z0190/
      DATA CODE( 1106),CODE( 1107),CODE( 1108),CODE( 1109)/
     *   Z0000,Z0009,Z0000,ZFEEC/
      DATA CODE( 1110),CODE( 1111),CODE( 1112),CODE( 1113)/
     *   Z0400,ZFEEB,Z0000,ZFEEC/
      DATA CODE( 1114),CODE( 1115),CODE( 1116),CODE( 1117)/
     *   Z0000,ZFEEC,Z0400,ZFEEC/
      DATA CODE( 1118),CODE( 1119),CODE( 1120),CODE( 1121)/
     *   Z0000,ZFEEC,Z0400,ZFEEC/
      DATA CODE( 1122),CODE( 1123),CODE( 1124),CODE( 1125)/
     *   Z0000,ZFEEC,Z0400,ZFEEC/
      DATA CODE( 1126),CODE( 1127),CODE( 1128),CODE( 1129)/
     *   Z120C,Z0000,Z0000,Z0161/
      DATA CODE( 1130),CODE( 1131),CODE( 1132),CODE( 1133)/
     *   Z120C,Z0000,Z0000,Z0082/
      DATA CODE( 1134),CODE( 1135),CODE( 1136),CODE( 1137)/
     *   Z120C,Z0000,Z0000,Z0188/
      DATA CODE( 1138),CODE( 1139),CODE( 1140),CODE( 1141)/
     *   Z0000,Z0001,Z0400,ZFEE5/
      DATA CODE( 1142),CODE( 1143),CODE( 1144),CODE( 1145)/
     *   Z4028,Z0000,Z0000,Z0000/
      DATA CODE( 1146),CODE( 1147),CODE( 1148),CODE( 1149)/
     *   Z4284,Z0000,Z0000,Z0000/
      DATA CODE( 1150),CODE( 1151),CODE( 1152),CODE( 1153)/
     *   Z120C,Z0000,Z0000,Z0184/
      DATA CODE( 1154),CODE( 1155),CODE( 1156),CODE( 1157)/
     *   Z0000,Z0001,Z0000,ZFEE0/
      DATA CODE( 1158),CODE( 1159),CODE( 1160),CODE( 1161)/
     *   Z120C,Z0000,Z0000,Z01A6/
      DATA CODE( 1162),CODE( 1163),CODE( 1164),CODE( 1165)/
     *   Z120C,Z0000,Z0000,Z0176/
      DATA CODE( 1166),CODE( 1167),CODE( 1168),CODE( 1169)/
     *   Z0000,ZFEDD,Z0000,Z0000/
      DATA CODE( 1170),CODE( 1171),CODE( 1172),CODE( 1173)/
     *   Z4284,Z0000,Z0000,Z0000/
      DATA CODE( 1174),CODE( 1175),CODE( 1176),CODE( 1177)/
     *   Z120C,Z0000,Z0000,Z017E/
      DATA CODE( 1178),CODE( 1179),CODE( 1180),CODE( 1181)/
     *   Z0000,Z0001,Z0400,ZFEDA/
      DATA CODE( 1182),CODE( 1183),CODE( 1184),CODE( 1185)/
     *   Z120C,Z0000,Z0000,Z01A0/
      DATA CODE( 1186),CODE( 1187),CODE( 1188),CODE( 1189)/
     *   Z120C,Z0000,Z0000,Z0170/
      DATA CODE( 1190),CODE( 1191),CODE( 1192),CODE( 1193)/
     *   Z0400,ZFED7,Z0000,Z0000/
      DATA CODE( 1194),CODE( 1195),CODE( 1196),CODE( 1197)/
     *   Z4284,Z0000,Z0000,Z0000/
      DATA CODE( 1198),CODE( 1199),CODE( 1200),CODE( 1201)/
     *   Z120C,Z0000,Z0000,Z0178/
      DATA CODE( 1202),CODE( 1203),CODE( 1204),CODE( 1205)/
     *   Z0000,Z0001,Z0000,ZFED5/
      DATA CODE( 1206),CODE( 1207),CODE( 1208),CODE( 1209)/
     *   Z120C,Z0000,Z0000,Z019A/
      DATA CODE( 1210),CODE( 1211),CODE( 1212),CODE( 1213)/
     *   Z120C,Z0000,Z0000,Z016A/
      DATA CODE( 1214),CODE( 1215),CODE( 1216),CODE( 1217)/
     *   Z0000,ZFED2,Z0000,Z0000/
      DATA CODE( 1218),CODE( 1219),CODE( 1220),CODE( 1221)/
     *   Z120C,Z0000,Z0000,Z017E/
      DATA CODE( 1222),CODE( 1223),CODE( 1224),CODE( 1225)/
     *   Z0000,ZFED4,Z0800,Z0001/
      DATA CODE( 1226),CODE( 1227),CODE( 1228),CODE( 1229)/
     *   Z120C,Z0000,Z0000,Z0196/
      DATA CODE( 1230),CODE( 1231),CODE( 1232),CODE( 1233)/
     *   Z120C,Z0000,Z0000,Z0165/
      DATA CODE( 1234),CODE( 1235),CODE( 1236),CODE( 1237)/
     *   Z0000,ZFED1,Z0000,Z0000/
      DATA CODE( 1238),CODE( 1239),CODE( 1240),CODE( 1241)/
     *   Z120C,Z0000,Z0000,Z0179/
      DATA CODE( 1242),CODE( 1243),CODE( 1244),CODE( 1245)/
     *   Z0000,ZFECF,Z0800,Z0000/
      DATA CODE( 1246),CODE( 1247),CODE( 1248),CODE( 1249)/
     *   Z120C,Z0000,Z0000,Z0191/
      DATA CODE( 1250),CODE( 1251),CODE( 1252),CODE( 1253)/
     *   Z1004,Z0192,Z0000,Z0000/
      DATA CODE( 1254),CODE( 1255),CODE( 1256),CODE( 1257)/
     *   Z1208,Z0000,Z0000,ZFFDA/
      DATA CODE( 1258),CODE( 1259),CODE( 1260),CODE( 1261)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE( 1262),CODE( 1263),CODE( 1264),CODE( 1265)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 1266),CODE( 1267),CODE( 1268),CODE( 1269)/
     *   Z0240,Z0000,Z0000,Z0030/
      DATA CODE( 1270),CODE( 1271),CODE( 1272),CODE( 1273)/
     *   Z0003,Z8600,Z0400,Z0917/
      DATA CODE( 1274),CODE( 1275),CODE( 1276),CODE( 1277)/
     *   Z03BC,Z0000,Z0400,Z0064/
      DATA CODE( 1278),CODE( 1279),CODE( 1280),CODE( 1281)/
     *   Z03B4,Z0000,Z0400,Z001B/
      DATA CODE( 1282),CODE( 1283),CODE( 1284),CODE( 1285)/
     *   Z4E38,Z0000,Z0000,Z0020/
      DATA CODE( 1286),CODE( 1287),CODE( 1288),CODE( 1289)/
     *   Z43FC,Z1800,Z4C07,Z0000/
      DATA CODE( 1290),CODE( 1291),CODE( 1292),CODE( 1293)/
     *   Z4045,ZC800,Z0000,Z0030/
      DATA CODE( 1294),CODE( 1295),CODE( 1296),CODE( 1297)/
     *   Z4375,ZDC00,Z9BC6,Z6000/
      DATA CODE( 1298),CODE( 1299),CODE( 1300),CODE( 1301)/
     *   Z028C,Z1800,Z2000,ZA000/
      DATA CODE( 1302),CODE( 1303),CODE( 1304),CODE( 1305)/
     *   Z0381,ZC600,Z879F,Z0000/
      DATA CODE( 1306),CODE( 1307),CODE( 1308),CODE( 1309)/
     *   Z4375,ZDC00,Z8182,Z0010/
      DATA CODE( 1310),CODE( 1311),CODE( 1312),CODE( 1313)/
     *   Z0279,Z8000,Z2000,Z6000/
      DATA CODE( 1314),CODE( 1315),CODE( 1316),CODE( 1317)/
     *   Z2380,ZB200,Z4A2C,Z0000/
      DATA CODE( 1318),CODE( 1319),CODE( 1320),CODE( 1321)/
     *   Z4239,Z8000,Z0000,Z0000/
      DATA CODE( 1322),CODE( 1323),CODE( 1324),CODE( 1325)/
     *   Z4088,Z0000,Z1BC0,Z9430/
      DATA CODE( 1326),CODE( 1327),CODE( 1328),CODE( 1329)/
     *   Z4375,ZDC00,Z0080,Z0000/
      DATA CODE( 1330),CODE( 1331),CODE( 1332),CODE( 1333)/
     *   Z31B9,Z8000,Z0000,Z0000/
      DATA CODE( 1334),CODE( 1335),CODE( 1336),CODE( 1337)/
     *   Z3038,ZB200,Z5A1D,ZA000/
      DATA CODE( 1338),CODE( 1339),CODE( 1340),CODE( 1341)/
     *   Z45A5,ZD7D4,Z0028,Z0000/
      DATA CODE( 1342),CODE( 1343),CODE( 1344),CODE( 1345)/
     *   Z23A4,Z0000,Z0000,Z0000/
      DATA CODE( 1346),CODE( 1347),CODE( 1348),CODE( 1349)/
     *   Z0000,Z01F2,Z0000,Z0000/
      DATA CODE( 1350),CODE( 1351),CODE( 1352),CODE( 1353)/
     *   Z1208,Z0000,Z0000,Z004A/
      DATA CODE( 1354),CODE( 1355),CODE( 1356),CODE( 1357)/
     *   Z4605,Z8000,Z01C0,Z1400/
      DATA CODE( 1358),CODE( 1359),CODE( 1360),CODE( 1361)/
     *   Z20C4,Z0000,Z0000,Z1F00/
      DATA CODE( 1362),CODE( 1363),CODE( 1364),CODE( 1365)/
     *   Z0000,Z0172,Z3000,ZFF00/
      DATA CODE( 1366),CODE( 1367),CODE( 1368),CODE( 1369)/
     *   Z1208,Z0000,Z0000,Z0046/
      DATA CODE( 1370),CODE( 1371),CODE( 1372),CODE( 1373)/
     *   Z0264,Z0000,Z0000,Z0000/
      DATA CODE( 1374),CODE( 1375),CODE( 1376),CODE( 1377)/
     *   Z0000,Z1600,ZC03F,Z0000/
      DATA CODE( 1378),CODE( 1379),CODE( 1380),CODE( 1381)/
     *   Z0000,Z1400,Z01C0,Z0000/
      DATA CODE( 1382),CODE( 1383),CODE( 1384),CODE( 1385)/
     *   Z0001,Z8000,Z2000,Z6000/
      DATA CODE( 1386),CODE( 1387),CODE( 1388),CODE( 1389)/
     *   Z03B4,Z0000,Z2818,Z4000/
      DATA CODE( 1390),CODE( 1391),CODE( 1392),CODE( 1393)/
     *   Z03B8,Z0000,Z0810,Z0000/
      DATA CODE( 1394),CODE( 1395),CODE( 1396),CODE( 1397)/
     *   Z2350,Z0000,Z0000,Z0000/
      DATA CODE( 1398),CODE( 1399),CODE( 1400),CODE( 1401)/
     *   Z2394,Z0000,Z0000,Z0000/
      DATA CODE( 1402),CODE( 1403),CODE( 1404),CODE( 1405)/
     *   Z23D4,Z0000,Z0000,Z0000/
      DATA CODE( 1406),CODE( 1407),CODE( 1408),CODE( 1409)/
     *   Z3180,Z0000,Z0000,Z0000/
      DATA CODE( 1410),CODE( 1411),CODE( 1412),CODE( 1413)/
     *   Z0280,Z0000,Z0000,Z0000/
      DATA CODE( 1414),CODE( 1415),CODE( 1416),CODE( 1417)/
     *   Z412C,Z01D9,Z0000,Z0000/
      DATA CODE( 1418),CODE( 1419),CODE( 1420),CODE( 1421)/
     *   Z2024,Z0000,Z0000,Z0000/
      DATA CODE( 1422),CODE( 1423),CODE( 1424),CODE( 1425)/
     *   Z0000,Z01F2,Z0000,Z0000/
      DATA CODE( 1426),CODE( 1427),CODE( 1428),CODE( 1429)/
     *   Z1208,Z0000,Z0000,Z0037/
      DATA CODE( 1430),CODE( 1431),CODE( 1432),CODE( 1433)/
     *   Z0238,Z0000,Z0000,Z0000/
      DATA CODE( 1434),CODE( 1435),CODE( 1436),CODE( 1437)/
     *   Z3038,Z0000,Z0000,Z0000/
      DATA CODE( 1438),CODE( 1439),CODE( 1440),CODE( 1441)/
     *   Z02B8,Z0000,Z0000,Z0000/
      DATA CODE( 1442),CODE( 1443),CODE( 1444),CODE( 1445)/
     *   Z23D0,Z01EF,Z0000,Z0000/
      DATA CODE( 1446),CODE( 1447),CODE( 1448),CODE( 1449)/
     *   Z412C,Z0054,Z0000,Z0000/
      DATA CODE( 1450),CODE( 1451),CODE( 1452),CODE( 1453)/
     *   Z2000,Z0000,Z0000,Z0000/
      DATA CODE( 1454),CODE( 1455),CODE( 1456),CODE( 1457)/
     *   Z2004,Z0000,Z0000,Z0000/
      DATA CODE( 1458),CODE( 1459),CODE( 1460),CODE( 1461)/
     *   Z200C,Z0000,Z0000,Z0000/
      DATA CODE( 1462),CODE( 1463),CODE( 1464),CODE( 1465)/
     *   Z03B4,Z0000,Z0400,Z001B/
      DATA CODE( 1466),CODE( 1467),CODE( 1468),CODE( 1469)/
     *   Z0380,Z0000,Z0400,Z0004/
      DATA CODE( 1470),CODE( 1471),CODE( 1472),CODE( 1473)/
     *   Z4264,Z0000,Z4C00,Z0000/
      DATA CODE( 1474),CODE( 1475),CODE( 1476),CODE( 1477)/
     *   Z4375,ZDDF2,Z0000,Z0000/
      DATA CODE( 1478),CODE( 1479),CODE( 1480),CODE( 1481)/
     *   Z1208,Z0000,Z0000,Z002A/
      DATA CODE( 1482),CODE( 1483),CODE( 1484),CODE( 1485)/
     *   Z03B5,Z8000,Z0400,Z0002/
      DATA CODE( 1486),CODE( 1487),CODE( 1488),CODE( 1489)/
     *   Z4633,Z8600,Z0400,Z0801/
      DATA CODE( 1490),CODE( 1491),CODE( 1492),CODE( 1493)/
     *   Z3270,Z0000,Z8000,Z0000/
      DATA CODE( 1494),CODE( 1495),CODE( 1496),CODE( 1497)/
     *   Z2330,Z0000,Z0000,Z0000/
      DATA CODE( 1498),CODE( 1499),CODE( 1500),CODE( 1501)/
     *   Z0270,Z0000,Z2000,Z0000/
      DATA CODE( 1502),CODE( 1503),CODE( 1504),CODE( 1505)/
     *   Z330C,Z0000,Z0000,Z0000/
      DATA CODE( 1506),CODE( 1507),CODE( 1508),CODE( 1509)/
     *   Z3304,Z0000,Z0000,Z0000/
      DATA CODE( 1510),CODE( 1511),CODE( 1512),CODE( 1513)/
     *   Z33D4,Z0000,Z0000,Z0000/
      DATA CODE( 1514),CODE( 1515),CODE( 1516),CODE( 1517)/
     *   Z23D0,Z0000,Z0000,Z0000/
      DATA CODE( 1518),CODE( 1519),CODE( 1520),CODE( 1521)/
     *   Z0244,Z0000,Z0000,Z0000/
      DATA CODE( 1522),CODE( 1523),CODE( 1524),CODE( 1525)/
     *   Z024C,Z0000,Z0000,Z0000/
      DATA CODE( 1526),CODE( 1527),CODE( 1528),CODE( 1529)/
     *   Z412C,Z0000,Z0000,Z0030/
      DATA CODE( 1530),CODE( 1531),CODE( 1532),CODE( 1533)/
     *   Z23D4,Z0000,Z0000,Z0030/
      DATA CODE( 1534),CODE( 1535),CODE( 1536),CODE( 1537)/
     *   Z2304,Z0000,Z0000,Z0030/
      DATA CODE( 1538),CODE( 1539),CODE( 1540),CODE( 1541)/
     *   Z0244,Z0000,Z0140,Z1730/
      DATA CODE( 1542),CODE( 1543),CODE( 1544),CODE( 1545)/
     *   Z230C,Z0000,Z4A03,Z1F30/
      DATA CODE( 1546),CODE( 1547),CODE( 1548),CODE( 1549)/
     *   Z0001,ZC800,Z1600,Z1F10/
      DATA CODE( 1550),CODE( 1551),CODE( 1552),CODE( 1553)/
     *   Z0000,ZC600,Z1AC0,Z7100/
      DATA CODE( 1554),CODE( 1555),CODE( 1556),CODE( 1557)/
     *   Z0285,Z8000,Z4A02,Z1F70/
      DATA CODE( 1558),CODE( 1559),CODE( 1560),CODE( 1561)/
     *   Z0000,Z0000,Z0000,Z1F00/
      DATA CODE( 1562),CODE( 1563),CODE( 1564),CODE( 1565)/
     *   Z334C,Z0172,ZE106,Z1100/
      DATA CODE( 1566),CODE( 1567),CODE( 1568),CODE( 1569)/
     *   Z1208,Z0000,Z0000,Z000B/
      DATA CODE( 1570),CODE( 1571),CODE( 1572),CODE( 1573)/
     *   Z23EC,Z0000,Z01A0,Z1630/
      DATA CODE( 1574),CODE( 1575),CODE( 1576),CODE( 1577)/
     *   Z4154,Z0000,Z1A00,Z5F30/
      DATA CODE( 1578),CODE( 1579),CODE( 1580),CODE( 1581)/
     *   Z2345,Z9400,Z0080,Z1F30/
      DATA CODE( 1582),CODE( 1583),CODE( 1584),CODE( 1585)/
     *   Z0245,Z9600,Z0150,Z1730/
      DATA CODE( 1586),CODE( 1587),CODE( 1588),CODE( 1589)/
     *   Z200D,ZA600,Z6B9B,Z3F30/
      DATA CODE( 1590),CODE( 1591),CODE( 1592),CODE( 1593)/
     *   Z0001,ZC800,Z8001,Z1F10/
      DATA CODE( 1594),CODE( 1595),CODE( 1596),CODE( 1597)/
     *   Z3344,ZC600,Z1AC0,Z7170/
      DATA CODE( 1598),CODE( 1599),CODE( 1600),CODE( 1601)/
     *   Z0245,Z8000,Z4A02,Z1F70/
      DATA CODE( 1602),CODE( 1603),CODE( 1604),CODE( 1605)/
     *   Z028C,Z0000,Z0640,Z1FF0/
      DATA CODE( 1606),CODE( 1607),CODE( 1608),CODE( 1609)/
     *   Z028C,Z0167,ZE90E,Z11F0/
      DATA CODE( 1610),CODE( 1611),CODE( 1612),CODE( 1613)/
     *   Z0000,Z0000,Z01A0,Z1600/
      DATA CODE( 1614),CODE( 1615),CODE( 1616),CODE( 1617)/
     *   Z0001,ZA600,Z0198,Z1F00/
      DATA CODE( 1618),CODE( 1619),CODE( 1620),CODE( 1621)/
     *   Z0001,Z9400,Z0080,Z1F00/
      DATA CODE( 1622),CODE( 1623),CODE( 1624),CODE( 1625)/
     *   Z0245,Z9800,Z0000,Z0070/
      DATA CODE( 1626),CODE( 1627),CODE( 1628),CODE( 1629)/
     *   Z234D,Z8000,Z0000,Z0070/
      DATA CODE( 1630),CODE( 1631),CODE( 1632),CODE( 1633)/
     *   Z029C,Z0000,Z0000,Z0000/
      DATA CODE( 1634),CODE( 1635),CODE( 1636),CODE( 1637)/
     *   Z024C,Z01F2,Z0000,Z0070/
      DATA CODE( 1638),CODE( 1639),CODE( 1640),CODE( 1641)/
     *   Z1208,Z0000,Z0000,Z0002/
      DATA CODE( 1642),CODE( 1643),CODE( 1644),CODE( 1645)/
     *   Z1208,Z0000,Z0000,ZFFE3/
      DATA CODE( 1646),CODE( 1647),CODE( 1648),CODE( 1649)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 1650),CODE( 1651),CODE( 1652),CODE( 1653)/
     *   Z0240,Z0000,Z0000,Z0030/
      DATA CODE( 1654),CODE( 1655),CODE( 1656),CODE( 1657)/
     *   Z0003,Z8600,Z0400,Z0917/
      DATA CODE( 1658),CODE( 1659),CODE( 1660),CODE( 1661)/
     *   Z03BC,Z0000,Z0400,Z0064/
      DATA CODE( 1662),CODE( 1663),CODE( 1664),CODE( 1665)/
     *   Z03B4,Z0000,Z0400,Z001B/
      DATA CODE( 1666),CODE( 1667),CODE( 1668),CODE( 1669)/
     *   Z4E38,Z0000,Z0000,Z0020/
      DATA CODE( 1670),CODE( 1671),CODE( 1672),CODE( 1673)/
     *   Z43FC,Z1800,Z4C07,Z0000/
      DATA CODE( 1674),CODE( 1675),CODE( 1676),CODE( 1677)/
     *   Z4045,ZC800,Z0000,Z0030/
      DATA CODE( 1678),CODE( 1679),CODE( 1680),CODE( 1681)/
     *   Z4375,ZDC00,Z9BC6,Z6000/
      DATA CODE( 1682),CODE( 1683),CODE( 1684),CODE( 1685)/
     *   Z0000,Z1600,Z2018,ZA000/
      DATA CODE( 1686),CODE( 1687),CODE( 1688),CODE( 1689)/
     *   Z0381,ZC600,Z879F,Z0000/
      DATA CODE( 1690),CODE( 1691),CODE( 1692),CODE( 1693)/
     *   Z4375,ZDC00,Z8182,Z0010/
      DATA CODE( 1694),CODE( 1695),CODE( 1696),CODE( 1697)/
     *   Z0279,Z8000,Z2000,Z6000/
      DATA CODE( 1698),CODE( 1699),CODE( 1700),CODE( 1701)/
     *   Z2380,ZB200,Z4A2C,Z0000/
      DATA CODE( 1702),CODE( 1703),CODE( 1704),CODE( 1705)/
     *   Z4239,Z8000,Z0000,Z0000/
      DATA CODE( 1706),CODE( 1707),CODE( 1708),CODE( 1709)/
     *   Z0000,Z0000,Z1BC0,Z9410/
      DATA CODE( 1710),CODE( 1711),CODE( 1712),CODE( 1713)/
     *   Z4375,ZDC00,Z0080,Z0000/
      DATA CODE( 1714),CODE( 1715),CODE( 1716),CODE( 1717)/
     *   Z31B9,Z8000,Z0000,Z0000/
      DATA CODE( 1718),CODE( 1719),CODE( 1720),CODE( 1721)/
     *   Z3038,ZB200,Z4A1D,Z0010/
      DATA CODE( 1722),CODE( 1723),CODE( 1724),CODE( 1725)/
     *   Z45A5,Z81D4,Z0000,Z0000/
      DATA CODE( 1726),CODE( 1727),CODE( 1728),CODE( 1729)/
     *   Z23A4,Z0000,Z0000,Z0000/
      DATA CODE( 1730),CODE( 1731),CODE( 1732),CODE( 1733)/
     *   Z0000,Z01F2,Z0000,Z0000/
      DATA CODE( 1734),CODE( 1735),CODE( 1736),CODE( 1737)/
     *   Z1208,Z0000,Z0000,Z004A/
      DATA CODE( 1738),CODE( 1739),CODE( 1740),CODE( 1741)/
     *   Z4605,Z8000,Z01C0,Z1400/
      DATA CODE( 1742),CODE( 1743),CODE( 1744),CODE( 1745)/
     *   Z20C4,Z0000,Z1A00,ZBF00/
      DATA CODE( 1746),CODE( 1747),CODE( 1748),CODE( 1749)/
     *   Z4088,Z0000,Z3000,ZFF30/
      DATA CODE( 1750),CODE( 1751),CODE( 1752),CODE( 1753)/
     *   Z0264,Z0000,Z0000,Z0000/
      DATA CODE( 1754),CODE( 1755),CODE( 1756),CODE( 1757)/
     *   Z0000,Z1600,ZC03F,Z0000/
      DATA CODE( 1758),CODE( 1759),CODE( 1760),CODE( 1761)/
     *   Z0000,Z1400,Z4BC3,Z0000/
      DATA CODE( 1762),CODE( 1763),CODE( 1764),CODE( 1765)/
     *   Z0001,ZD800,Z2000,Z6000/
      DATA CODE( 1766),CODE( 1767),CODE( 1768),CODE( 1769)/
     *   Z03B5,Z8000,Z2818,Z4000/
      DATA CODE( 1770),CODE( 1771),CODE( 1772),CODE( 1773)/
     *   Z03B8,Z0000,Z0810,Z0000/
      DATA CODE( 1774),CODE( 1775),CODE( 1776),CODE( 1777)/
     *   Z2350,Z0172,Z0000,Z0000/
      DATA CODE( 1778),CODE( 1779),CODE( 1780),CODE( 1781)/
     *   Z1208,Z0000,Z0000,Z003F/
      DATA CODE( 1782),CODE( 1783),CODE( 1784),CODE( 1785)/
     *   Z2394,Z0000,Z0000,Z0000/
      DATA CODE( 1786),CODE( 1787),CODE( 1788),CODE( 1789)/
     *   Z23D4,Z0000,Z0000,Z0000/
      DATA CODE( 1790),CODE( 1791),CODE( 1792),CODE( 1793)/
     *   Z3180,Z0000,Z0000,Z0000/
      DATA CODE( 1794),CODE( 1795),CODE( 1796),CODE( 1797)/
     *   Z0280,Z0000,Z0000,Z0000/
      DATA CODE( 1798),CODE( 1799),CODE( 1800),CODE( 1801)/
     *   Z412C,Z01D9,Z0000,Z0000/
      DATA CODE( 1802),CODE( 1803),CODE( 1804),CODE( 1805)/
     *   Z2024,Z0000,Z0000,Z0000/
      DATA CODE( 1806),CODE( 1807),CODE( 1808),CODE( 1809)/
     *   Z0000,Z01F2,Z0000,Z0000/
      DATA CODE( 1810),CODE( 1811),CODE( 1812),CODE( 1813)/
     *   Z1208,Z0000,Z0000,Z0037/
      DATA CODE( 1814),CODE( 1815),CODE( 1816),CODE( 1817)/
     *   Z0238,Z0000,Z0000,Z0000/
      DATA CODE( 1818),CODE( 1819),CODE( 1820),CODE( 1821)/
     *   Z3038,Z0000,Z0000,Z0000/
      DATA CODE( 1822),CODE( 1823),CODE( 1824),CODE( 1825)/
     *   Z02B8,Z0000,Z0000,Z0000/
      DATA CODE( 1826),CODE( 1827),CODE( 1828),CODE( 1829)/
     *   Z23D0,Z01EF,Z0000,Z0000/
      DATA CODE( 1830),CODE( 1831),CODE( 1832),CODE( 1833)/
     *   Z412C,Z0054,Z0000,Z0000/
      DATA CODE( 1834),CODE( 1835),CODE( 1836),CODE( 1837)/
     *   Z2000,Z0000,Z0000,Z0000/
      DATA CODE( 1838),CODE( 1839),CODE( 1840),CODE( 1841)/
     *   Z200C,Z0000,Z0000,Z0000/
      DATA CODE( 1842),CODE( 1843),CODE( 1844),CODE( 1845)/
     *   Z2004,Z0000,Z0000,Z0000/
      DATA CODE( 1846),CODE( 1847),CODE( 1848),CODE( 1849)/
     *   Z0380,Z0000,Z0400,Z0004/
      DATA CODE( 1850),CODE( 1851),CODE( 1852),CODE( 1853)/
     *   Z03B4,Z0000,Z0400,Z0002/
      DATA CODE( 1854),CODE( 1855),CODE( 1856),CODE( 1857)/
     *   Z4630,Z0000,Z0000,Z0000/
      DATA CODE( 1858),CODE( 1859),CODE( 1860),CODE( 1861)/
     *   Z3270,Z0000,Z0000,Z0000/
      DATA CODE( 1862),CODE( 1863),CODE( 1864),CODE( 1865)/
     *   Z0270,Z0000,Z0000,Z0000/
      DATA CODE( 1866),CODE( 1867),CODE( 1868),CODE( 1869)/
     *   Z2330,Z0000,Z0000,Z0000/
      DATA CODE( 1870),CODE( 1871),CODE( 1872),CODE( 1873)/
     *   Z330C,Z0000,Z0000,Z0000/
      DATA CODE( 1874),CODE( 1875),CODE( 1876),CODE( 1877)/
     *   Z3304,Z0000,Z0000,Z0000/
      DATA CODE( 1878),CODE( 1879),CODE( 1880),CODE( 1881)/
     *   Z33D4,Z0000,Z0000,Z0000/
      DATA CODE( 1882),CODE( 1883),CODE( 1884),CODE( 1885)/
     *   Z23D0,Z0000,Z0000,Z0000/
      DATA CODE( 1886),CODE( 1887),CODE( 1888),CODE( 1889)/
     *   Z412C,Z0000,Z0000,Z0030/
      DATA CODE( 1890),CODE( 1891),CODE( 1892),CODE( 1893)/
     *   Z23D4,Z0000,Z0000,Z0030/
      DATA CODE( 1894),CODE( 1895),CODE( 1896),CODE( 1897)/
     *   Z230C,Z0000,Z0000,Z0030/
      DATA CODE( 1898),CODE( 1899),CODE( 1900),CODE( 1901)/
     *   Z0000,Z0000,Z00C0,Z1710/
      DATA CODE( 1902),CODE( 1903),CODE( 1904),CODE( 1905)/
     *   Z2304,Z0000,Z4A02,Z1F30/
      DATA CODE( 1906),CODE( 1907),CODE( 1908),CODE( 1909)/
     *   Z4268,Z0000,Z4A06,Z1F10/
      DATA CODE( 1910),CODE( 1911),CODE( 1912),CODE( 1913)/
     *   Z334C,Z0000,Z1A80,Z3100/
      DATA CODE( 1914),CODE( 1915),CODE( 1916),CODE( 1917)/
     *   Z3344,Z0000,Z4A00,Z1F00/
      DATA CODE( 1918),CODE( 1919),CODE( 1920),CODE( 1921)/
     *   Z02A8,Z0000,Z4A01,Z1F00/
      DATA CODE( 1922),CODE( 1923),CODE( 1924),CODE( 1925)/
     *   Z0001,Z9BF2,ZC102,Z1100/
      DATA CODE( 1926),CODE( 1927),CODE( 1928),CODE( 1929)/
     *   Z1208,Z0000,Z0000,Z000D/
      DATA CODE( 1930),CODE( 1931),CODE( 1932),CODE( 1933)/
     *   Z23ED,Z8000,Z00A0,Z1930/
      DATA CODE( 1934),CODE( 1935),CODE( 1936),CODE( 1937)/
     *   Z4154,Z0000,Z0140,Z1430/
      DATA CODE( 1938),CODE( 1939),CODE( 1940),CODE( 1941)/
     *   Z200D,Z9400,Z01A8,Z1830/
      DATA CODE( 1942),CODE( 1943),CODE( 1944),CODE( 1945)/
     *   Z0001,Z9600,Z00C8,Z1710/
      DATA CODE( 1946),CODE( 1947),CODE( 1948),CODE( 1949)/
     *   Z2005,Z9400,Z6A02,ZDF30/
      DATA CODE( 1950),CODE( 1951),CODE( 1952),CODE( 1953)/
     *   Z0001,Z9400,Z6A46,ZFF10/
      DATA CODE( 1954),CODE( 1955),CODE( 1956),CODE( 1957)/
     *   Z3345,ZD600,Z1AB0,Z3170/
      DATA CODE( 1958),CODE( 1959),CODE( 1960),CODE( 1961)/
     *   Z02A9,ZD600,Z4A38,Z1F50/
      DATA CODE( 1962),CODE( 1963),CODE( 1964),CODE( 1965)/
     *   Z334D,Z81F2,Z4A01,Z1F70/
      DATA CODE( 1966),CODE( 1967),CODE( 1968),CODE( 1969)/
     *   Z0000,Z0052,Z0000,Z0000/
      DATA CODE( 1970),CODE( 1971),CODE( 1972),CODE( 1973)/
     *   Z0001,Z9A46,ZC102,Z1150/
      DATA CODE( 1974),CODE( 1975),CODE( 1976),CODE( 1977)/
     *   Z0001,Z9A00,ZC102,Z1150/
      DATA CODE( 1978),CODE( 1979),CODE( 1980),CODE( 1981)/
     *   Z0001,Z8000,Z00A0,Z1900/
      DATA CODE( 1982),CODE( 1983),CODE( 1984),CODE( 1985)/
     *   Z0000,Z0000,Z0140,Z1400/
      DATA CODE( 1986),CODE( 1987),CODE( 1988),CODE( 1989)/
     *   Z0001,Z9400,Z01A8,Z1800/
      DATA CODE( 1990),CODE( 1991),CODE( 1992),CODE( 1993)/
     *   Z0001,Z9600,Z0008,Z1F00/
      DATA CODE( 1994),CODE( 1995),CODE( 1996),CODE( 1997)/
     *   Z0001,Z9400,Z8006,Z1F00/
      DATA CODE( 1998),CODE( 1999),CODE( 2000),CODE( 2001)/
     *   Z0001,Z9400,Z8047,Z0000/
      DATA CODE( 2002),CODE( 2003),CODE( 2004),CODE( 2005)/
     *   Z234C,Z0000,Z0780,Z00F0/
      DATA CODE( 2006),CODE( 2007),CODE( 2008),CODE( 2009)/
     *   Z0000,Z0000,Z07C0,Z00D0/
      DATA CODE( 2010),CODE( 2011),CODE( 2012),CODE( 2013)/
     *   Z2345,Z8000,Z0000,Z0070/
      DATA CODE( 2014),CODE( 2015),CODE( 2016),CODE( 2017)/
     *   Z029C,Z0000,Z0000,Z0000/
      DATA CODE( 2018),CODE( 2019),CODE( 2020),CODE( 2021)/
     *   Z0000,Z01F2,Z0000,Z0050/
      DATA CODE( 2022),CODE( 2023),CODE( 2024),CODE( 2025)/
     *   Z1208,Z0000,Z0000,Z0002/
      DATA CODE( 2026),CODE( 2027),CODE( 2028),CODE( 2029)/
     *   Z1208,Z0000,Z0000,ZFFDD/
      DATA CODE( 2030),CODE( 2031),CODE( 2032),CODE( 2033)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2034),CODE( 2035),CODE( 2036),CODE( 2037)/
     *   Z03BC,Z0000,Z0400,Z006A/
      DATA CODE( 2038),CODE( 2039),CODE( 2040),CODE( 2041)/
     *   Z120C,Z0000,Z0000,Z00CC/
      DATA CODE( 2042),CODE( 2043),CODE( 2044),CODE( 2045)/
     *   Z41BC,Z0000,Z0000,Z0000/
      DATA CODE( 2046),CODE( 2047),CODE( 2048),CODE( 2049)/
     *   Z120C,Z0000,Z0000,Z007F/
      DATA CODE( 2050),CODE( 2051),CODE( 2052),CODE( 2053)/
     *   Z4000,Z0119,Z0000,Z0030/
      DATA CODE( 2054),CODE( 2055),CODE( 2056),CODE( 2057)/
     *   Z4088,Z0000,Z0000,Z0030/
      DATA CODE( 2058),CODE( 2059),CODE( 2060),CODE( 2061)/
     *   Z3150,Z0000,Z0000,Z0000/
      DATA CODE( 2062),CODE( 2063),CODE( 2064),CODE( 2065)/
     *   Z2040,Z0000,Z4A04,Z0030/
      DATA CODE( 2066),CODE( 2067),CODE( 2068),CODE( 2069)/
     *   Z20C8,ZC400,Z1B40,Z8030/
      DATA CODE( 2070),CODE( 2071),CODE( 2072),CODE( 2073)/
     *   Z0001,ZA600,Z0120,Z0000/
      DATA CODE( 2074),CODE( 2075),CODE( 2076),CODE( 2077)/
     *   Z2040,Z0000,Z4A04,Z0030/
      DATA CODE( 2078),CODE( 2079),CODE( 2080),CODE( 2081)/
     *   Z20C8,ZC200,Z1A00,Z8030/
      DATA CODE( 2082),CODE( 2083),CODE( 2084),CODE( 2085)/
     *   Z2151,ZA76E,Z0120,Z0070/
      DATA CODE( 2086),CODE( 2087),CODE( 2088),CODE( 2089)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2090),CODE( 2091),CODE( 2092),CODE( 2093)/
     *   Z03BC,Z0000,Z0400,Z006A/
      DATA CODE( 2094),CODE( 2095),CODE( 2096),CODE( 2097)/
     *   Z120C,Z0000,Z0000,Z00BE/
      DATA CODE( 2098),CODE( 2099),CODE( 2100),CODE( 2101)/
     *   Z41BC,Z0000,Z0000,Z0000/
      DATA CODE( 2102),CODE( 2103),CODE( 2104),CODE( 2105)/
     *   Z120C,Z0000,Z0000,Z0071/
      DATA CODE( 2106),CODE( 2107),CODE( 2108),CODE( 2109)/
     *   Z4000,Z0119,Z0000,Z0030/
      DATA CODE( 2110),CODE( 2111),CODE( 2112),CODE( 2113)/
     *   Z4088,Z0000,Z0000,Z0030/
      DATA CODE( 2114),CODE( 2115),CODE( 2116),CODE( 2117)/
     *   Z3150,Z0000,Z0000,Z0000/
      DATA CODE( 2118),CODE( 2119),CODE( 2120),CODE( 2121)/
     *   Z2040,ZC400,Z4B44,Z0030/
      DATA CODE( 2122),CODE( 2123),CODE( 2124),CODE( 2125)/
     *   Z20C8,Z0000,Z0100,Z1730/
      DATA CODE( 2126),CODE( 2127),CODE( 2128),CODE( 2129)/
     *   Z0000,ZC200,Z0000,Z1F00/
      DATA CODE( 2130),CODE( 2131),CODE( 2132),CODE( 2133)/
     *   Z2041,Z8000,Z4A04,Z1F30/
      DATA CODE( 2134),CODE( 2135),CODE( 2136),CODE( 2137)/
     *   Z20C8,ZC200,Z3100,Z9730/
      DATA CODE( 2138),CODE( 2139),CODE( 2140),CODE( 2141)/
     *   Z2150,Z016E,Z0820,Z1FF0/
      DATA CODE( 2142),CODE( 2143),CODE( 2144),CODE( 2145)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2146),CODE( 2147),CODE( 2148),CODE( 2149)/
     *   Z03BC,Z0000,Z0400,Z0032/
      DATA CODE( 2150),CODE( 2151),CODE( 2152),CODE( 2153)/
     *   Z120C,Z0000,Z0000,Z00B0/
      DATA CODE( 2154),CODE( 2155),CODE( 2156),CODE( 2157)/
     *   Z417C,Z0000,Z0000,Z0000/
      DATA CODE( 2158),CODE( 2159),CODE( 2160),CODE( 2161)/
     *   Z120C,Z0000,Z0000,Z0063/
      DATA CODE( 2162),CODE( 2163),CODE( 2164),CODE( 2165)/
     *   Z4088,Z0119,Z0000,Z0030/
      DATA CODE( 2166),CODE( 2167),CODE( 2168),CODE( 2169)/
     *   Z4000,Z0000,Z0000,Z0030/
      DATA CODE( 2170),CODE( 2171),CODE( 2172),CODE( 2173)/
     *   Z310C,Z0000,Z0000,Z0000/
      DATA CODE( 2174),CODE( 2175),CODE( 2176),CODE( 2177)/
     *   Z2040,Z0000,Z4A04,Z0030/
      DATA CODE( 2178),CODE( 2179),CODE( 2180),CODE( 2181)/
     *   Z0000,Z0000,Z0100,Z1700/
      DATA CODE( 2182),CODE( 2183),CODE( 2184),CODE( 2185)/
     *   Z2040,ZC400,Z0140,Z1F30/
      DATA CODE( 2186),CODE( 2187),CODE( 2188),CODE( 2189)/
     *   Z0001,Z8000,Z0100,Z1700/
      DATA CODE( 2190),CODE( 2191),CODE( 2192),CODE( 2193)/
     *   Z2040,ZC200,Z3000,Z9F30/
      DATA CODE( 2194),CODE( 2195),CODE( 2196),CODE( 2197)/
     *   Z210D,Z816F,Z0920,Z17F0/
      DATA CODE( 2198),CODE( 2199),CODE( 2200),CODE( 2201)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2202),CODE( 2203),CODE( 2204),CODE( 2205)/
     *   Z03BC,Z0000,Z0400,Z001A/
      DATA CODE( 2206),CODE( 2207),CODE( 2208),CODE( 2209)/
     *   Z120C,Z0000,Z0000,Z00A2/
      DATA CODE( 2210),CODE( 2211),CODE( 2212),CODE( 2213)/
     *   Z4000,Z0000,Z0000,Z0030/
      DATA CODE( 2214),CODE( 2215),CODE( 2216),CODE( 2217)/
     *   Z30C8,Z0000,Z0000,Z0000/
      DATA CODE( 2218),CODE( 2219),CODE( 2220),CODE( 2221)/
     *   Z2040,Z0000,Z0000,Z0030/
      DATA CODE( 2222),CODE( 2223),CODE( 2224),CODE( 2225)/
     *   Z4110,Z0000,Z4A04,Z0000/
      DATA CODE( 2226),CODE( 2227),CODE( 2228),CODE( 2229)/
     *   Z0000,Z0194,Z0100,Z1500/
      DATA CODE( 2230),CODE( 2231),CODE( 2232),CODE( 2233)/
     *   Z2040,Z0000,Z4A04,Z1F30/
      DATA CODE( 2234),CODE( 2235),CODE( 2236),CODE( 2237)/
     *   Z0290,Z0000,Z0100,Z1500/
      DATA CODE( 2238),CODE( 2239),CODE( 2240),CODE( 2241)/
     *   Z20C8,Z01AE,Z0000,Z00B0/
      DATA CODE( 2242),CODE( 2243),CODE( 2244),CODE( 2245)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2246),CODE( 2247),CODE( 2248),CODE( 2249)/
     *   Z03BC,Z0000,Z0400,Z001A/
      DATA CODE( 2250),CODE( 2251),CODE( 2252),CODE( 2253)/
     *   Z120C,Z0000,Z0000,Z0097/
      DATA CODE( 2254),CODE( 2255),CODE( 2256),CODE( 2257)/
     *   Z4113,Z8600,Z0400,Z0801/
      DATA CODE( 2258),CODE( 2259),CODE( 2260),CODE( 2261)/
     *   Z4000,Z01B2,Z0000,Z0030/
      DATA CODE( 2262),CODE( 2263),CODE( 2264),CODE( 2265)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2266),CODE( 2267),CODE( 2268),CODE( 2269)/
     *   Z30CB,Z8600,Z0400,Z08CF/
      DATA CODE( 2270),CODE( 2271),CODE( 2272),CODE( 2273)/
     *   Z0337,Z9600,Z4E01,Z0001/
      DATA CODE( 2274),CODE( 2275),CODE( 2276),CODE( 2277)/
     *   Z0238,Z0000,Z4A04,Z1F01/
      DATA CODE( 2278),CODE( 2279),CODE( 2280),CODE( 2281)/
     *   Z0279,Z4400,Z0100,Z1F01/
      DATA CODE( 2282),CODE( 2283),CODE( 2284),CODE( 2285)/
     *   Z0001,Z8000,Z1E00,Z9F01/
      DATA CODE( 2286),CODE( 2287),CODE( 2288),CODE( 2289)/
     *   Z0002,Z9600,Z1E20,Z8001/
      DATA CODE( 2290),CODE( 2291),CODE( 2292),CODE( 2293)/
     *   Z0002,Z8774,Z1E20,Z8001/
      DATA CODE( 2294),CODE( 2295),CODE( 2296),CODE( 2297)/
     *   Z0001,Z2600,Z0120,Z1C01/
      DATA CODE( 2298),CODE( 2299),CODE( 2300),CODE( 2301)/
     *   Z0000,Z0000,Z4404,Z03FF/
      DATA CODE( 2302),CODE( 2303),CODE( 2304),CODE( 2305)/
     *   Z0001,ZDA53,Z1700,Z9501/
      DATA CODE( 2306),CODE( 2307),CODE( 2308),CODE( 2309)/
     *   Z0001,Z2600,Z8124,Z1C01/
      DATA CODE( 2310),CODE( 2311),CODE( 2312),CODE( 2313)/
     *   Z0000,Z1200,Z1F00,Z9501/
      DATA CODE( 2314),CODE( 2315),CODE( 2316),CODE( 2317)/
     *   Z0001,Z8000,Z1F20,Z9901/
      DATA CODE( 2318),CODE( 2319),CODE( 2320),CODE( 2321)/
     *   Z0001,Z9774,Z9E23,Z9F01/
      DATA CODE( 2322),CODE( 2323),CODE( 2324),CODE( 2325)/
     *   Z0001,Z8000,Z5025,Z9200/
      DATA CODE( 2326),CODE( 2327),CODE( 2328),CODE( 2329)/
     *   Z12CC,Z0000,Z4004,Z0010/
      DATA CODE( 2330),CODE( 2331),CODE( 2332),CODE( 2333)/
     *   Z0000,Z0052,Z4403,ZFDFF/
      DATA CODE( 2334),CODE( 2335),CODE( 2336),CODE( 2337)/
     *   Z0001,Z8000,ZDE25,Z9200/
      DATA CODE( 2338),CODE( 2339),CODE( 2340),CODE( 2341)/
     *   Z0001,Z9600,Z2120,Z9D01/
      DATA CODE( 2342),CODE( 2343),CODE( 2344),CODE( 2345)/
     *   Z2041,Z8000,Z0140,Z1530/
      DATA CODE( 2346),CODE( 2347),CODE( 2348),CODE( 2349)/
     *   Z4375,Z9600,Z1F60,Z9403/
      DATA CODE( 2350),CODE( 2351),CODE( 2352),CODE( 2353)/
     *   Z03BD,Z9600,Z06E0,Z0001/
      DATA CODE( 2354),CODE( 2355),CODE( 2356),CODE( 2357)/
     *   Z0001,Z8000,Z6A04,Z9F01/
      DATA CODE( 2358),CODE( 2359),CODE( 2360),CODE( 2361)/
     *   Z027D,Z4400,Z6D26,Z9201/
      DATA CODE( 2362),CODE( 2363),CODE( 2364),CODE( 2365)/
     *   Z0001,Z9600,Z1E20,Z9F01/
      DATA CODE( 2366),CODE( 2367),CODE( 2368),CODE( 2369)/
     *   Z0002,Z9600,Z1E20,Z9F01/
      DATA CODE( 2370),CODE( 2371),CODE( 2372),CODE( 2373)/
     *   Z0002,Z8774,ZDE62,Z9401/
      DATA CODE( 2374),CODE( 2375),CODE( 2376),CODE( 2377)/
     *   Z0000,ZA600,Z0120,Z1C01/
      DATA CODE( 2378),CODE( 2379),CODE( 2380),CODE( 2381)/
     *   Z0000,Z0000,Z4404,Z03FF/
      DATA CODE( 2382),CODE( 2383),CODE( 2384),CODE( 2385)/
     *   Z0001,ZDA57,Z1700,Z9501/
      DATA CODE( 2386),CODE( 2387),CODE( 2388),CODE( 2389)/
     *   Z0000,ZA655,Z8124,Z1C01/
      DATA CODE( 2390),CODE( 2391),CODE( 2392),CODE( 2393)/
     *   Z0000,Z0020,Z1A3A,ZAE0B/
      DATA CODE( 2394),CODE( 2395),CODE( 2396),CODE( 2397)/
     *   Z0001,ZA200,Z50A5,Z9200/
      DATA CODE( 2398),CODE( 2399),CODE( 2400),CODE( 2401)/
     *   Z12CC,Z0000,Z4004,ZFFFE/
      DATA CODE( 2402),CODE( 2403),CODE( 2404),CODE( 2405)/
     *   Z0000,Z0055,Z4403,ZFDFF/
      DATA CODE( 2406),CODE( 2407),CODE( 2408),CODE( 2409)/
     *   Z0000,Z1200,Z1F00,Z9501/
      DATA CODE( 2410),CODE( 2411),CODE( 2412),CODE( 2413)/
     *   Z0001,Z9A00,Z1F20,Z9901/
      DATA CODE( 2414),CODE( 2415),CODE( 2416),CODE( 2417)/
     *   Z0001,Z976B,Z9E23,Z9F01/
      DATA CODE( 2418),CODE( 2419),CODE( 2420),CODE( 2421)/
     *   Z0001,ZA200,ZDEA5,Z9200/
      DATA CODE( 2422),CODE( 2423),CODE( 2424),CODE( 2425)/
     *   Z0001,Z9600,Z2120,Z9D01/
      DATA CODE( 2426),CODE( 2427),CODE( 2428),CODE( 2429)/
     *   Z2041,Z8000,Z8142,Z1530/
      DATA CODE( 2430),CODE( 2431),CODE( 2432),CODE( 2433)/
     *   Z4375,Z9600,Z1F60,Z9403/
      DATA CODE( 2434),CODE( 2435),CODE( 2436),CODE( 2437)/
     *   Z03BD,Z9600,Z06E0,Z0001/
      DATA CODE( 2438),CODE( 2439),CODE( 2440),CODE( 2441)/
     *   Z43B9,ZDE00,Z6B84,Z9F01/
      DATA CODE( 2442),CODE( 2443),CODE( 2444),CODE( 2445)/
     *   Z027C,ZC400,Z6D26,Z9201/
      DATA CODE( 2446),CODE( 2447),CODE( 2448),CODE( 2449)/
     *   Z0001,Z9600,Z1EA0,Z9401/
      DATA CODE( 2450),CODE( 2451),CODE( 2452),CODE( 2453)/
     *   Z0002,Z9600,Z1E20,Z9F01/
      DATA CODE( 2454),CODE( 2455),CODE( 2456),CODE( 2457)/
     *   Z0292,Z8773,ZDE62,Z9401/
      DATA CODE( 2458),CODE( 2459),CODE( 2460),CODE( 2461)/
     *   Z20C8,ZA7A3,Z8124,Z1CB1/
      DATA CODE( 2462),CODE( 2463),CODE( 2464),CODE( 2465)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2466),CODE( 2467),CODE( 2468),CODE( 2469)/
     *   Z20C8,ZA78F,Z0120,Z1CB1/
      DATA CODE( 2470),CODE( 2471),CODE( 2472),CODE( 2473)/
     *   Z0000,Z0000,Z4404,Z03FF/
      DATA CODE( 2474),CODE( 2475),CODE( 2476),CODE( 2477)/
     *   Z0001,ZDA40,Z1700,Z9501/
      DATA CODE( 2478),CODE( 2479),CODE( 2480),CODE( 2481)/
     *   Z03BC,Z0000,Z0400,Z001A/
      DATA CODE( 2482),CODE( 2483),CODE( 2484),CODE( 2485)/
     *   Z120C,Z0000,Z0000,Z005D/
      DATA CODE( 2486),CODE( 2487),CODE( 2488),CODE( 2489)/
     *   Z03B4,Z0000,Z0400,Z001B/
      DATA CODE( 2490),CODE( 2491),CODE( 2492),CODE( 2493)/
     *   Z4000,Z0000,Z0000,Z0030/
      DATA CODE( 2494),CODE( 2495),CODE( 2496),CODE( 2497)/
     *   Z30CB,Z8600,Z0400,Z0801/
      DATA CODE( 2498),CODE( 2499),CODE( 2500),CODE( 2501)/
     *   Z2040,Z0000,Z0000,Z0030/
      DATA CODE( 2502),CODE( 2503),CODE( 2504),CODE( 2505)/
     *   Z0000,Z0000,Z1A00,Z8000/
      DATA CODE( 2506),CODE( 2507),CODE( 2508),CODE( 2509)/
     *   Z4110,Z0000,Z4C05,Z0000/
      DATA CODE( 2510),CODE( 2511),CODE( 2512),CODE( 2513)/
     *   Z4375,ZDD96,Z4964,Z0000/
      DATA CODE( 2514),CODE( 2515),CODE( 2516),CODE( 2517)/
     *   Z2041,Z8000,Z1A00,Z8030/
      DATA CODE( 2518),CODE( 2519),CODE( 2520),CODE( 2521)/
     *   Z0000,ZC200,Z0000,Z0000/
      DATA CODE( 2522),CODE( 2523),CODE( 2524),CODE( 2525)/
     *   Z4375,ZDC00,Z4924,Z0000/
      DATA CODE( 2526),CODE( 2527),CODE( 2528),CODE( 2529)/
     *   Z2040,ZC200,Z1A00,Z8030/
      DATA CODE( 2530),CODE( 2531),CODE( 2532),CODE( 2533)/
     *   Z20C8,Z016E,Z0000,Z0070/
      DATA CODE( 2534),CODE( 2535),CODE( 2536),CODE( 2537)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2538),CODE( 2539),CODE( 2540),CODE( 2541)/
     *   Z0000,Z0000,Z4004,Z0000/
      DATA CODE( 2542),CODE( 2543),CODE( 2544),CODE( 2545)/
     *   Z0001,ZA400,Z0100,Z1500/
      DATA CODE( 2546),CODE( 2547),CODE( 2548),CODE( 2549)/
     *   Z0001,ZA400,Z0100,Z1500/
      DATA CODE( 2550),CODE( 2551),CODE( 2552),CODE( 2553)/
     *   Z0003,Z8CE0,Z0100,Z1500/
      DATA CODE( 2554),CODE( 2555),CODE( 2556),CODE( 2557)/
     *   Z43FC,Z0000,Z4C05,Z0000/
      DATA CODE( 2558),CODE( 2559),CODE( 2560),CODE( 2561)/
     *   Z03BC,Z01D3,Z0400,Z001B/
      DATA CODE( 2562),CODE( 2563),CODE( 2564),CODE( 2565)/
     *   Z12CC,Z0000,Z1004,Z0005/
      DATA CODE( 2566),CODE( 2567),CODE( 2568),CODE( 2569)/
     *   Z43FD,ZBC52,Z0160,Z0000/
      DATA CODE( 2570),CODE( 2571),CODE( 2572),CODE( 2573)/
     *   Z43FD,ZDC00,Z0140,Z0000/
      DATA CODE( 2574),CODE( 2575),CODE( 2576),CODE( 2577)/
     *   Z0001,Z8000,Z0000,Z0000/
      DATA CODE( 2578),CODE( 2579),CODE( 2580),CODE( 2581)/
     *   Z0003,Z86E0,Z8405,Z0801/
      DATA CODE( 2582),CODE( 2583),CODE( 2584),CODE( 2585)/
     *   Z0000,Z0021,Z1400,Z0000/
      DATA CODE( 2586),CODE( 2587),CODE( 2588),CODE( 2589)/
     *   Z033F,ZB000,Z0000,Z0000/
      DATA CODE( 2590),CODE( 2591),CODE( 2592),CODE( 2593)/
     *   Z43FC,Z0000,Z0000,Z0003/
      DATA CODE( 2594),CODE( 2595),CODE( 2596),CODE( 2597)/
     *   Z12D0,Z0000,Z4004,Z0000/
      DATA CODE( 2598),CODE( 2599),CODE( 2600),CODE( 2601)/
     *   Z03B4,Z0000,Z0700,Z0000/
      DATA CODE( 2602),CODE( 2603),CODE( 2604),CODE( 2605)/
     *   Z12D4,Z0000,Z4004,Z0001/
      DATA CODE( 2606),CODE( 2607),CODE( 2608),CODE( 2609)/
     *   Z03B8,Z0000,Z0700,Z0000/
      DATA CODE( 2610),CODE( 2611),CODE( 2612),CODE( 2613)/
     *   Z1314,Z0000,Z0000,Z0000/
      DATA CODE( 2614),CODE( 2615),CODE( 2616),CODE( 2617)/
     *   Z23BC,Z0000,Z0000,Z0003/
      DATA CODE( 2618),CODE( 2619),CODE( 2620),CODE( 2621)/
     *   Z03B8,Z0000,Z0400,ZFFFF/
      DATA CODE( 2622),CODE( 2623),CODE( 2624),CODE( 2625)/
     *   Z0E74,Z0000,Z0000,Z0000/
      DATA CODE( 2626),CODE( 2627),CODE( 2628),CODE( 2629)/
     *   Z027B,Z8397,Z0C00,Z0000/
      DATA CODE( 2630),CODE( 2631),CODE( 2632),CODE( 2633)/
     *   Z0380,Z0000,Z4C04,Z0000/
      DATA CODE( 2634),CODE( 2635),CODE( 2636),CODE( 2637)/
     *   Z12F0,Z0000,Z0700,Z0000/
      DATA CODE( 2638),CODE( 2639),CODE( 2640),CODE( 2641)/
     *   Z027B,Z8200,Z0C00,Z0000/
      DATA CODE( 2642),CODE( 2643),CODE( 2644),CODE( 2645)/
     *   Z0380,Z0000,Z4C04,Z0000/
      DATA CODE( 2646),CODE( 2647),CODE( 2648),CODE( 2649)/
     *   Z12F4,Z0000,Z0700,Z0001/
      DATA CODE( 2650),CODE( 2651),CODE( 2652),CODE( 2653)/
     *   Z02B4,Z004A,Z0000,Z0000/
      DATA CODE( 2654),CODE( 2655),CODE( 2656),CODE( 2657)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2658),CODE( 2659),CODE( 2660),CODE( 2661)/
     *   Z4000,Z0000,Z1C00,Z8000/
      DATA CODE( 2662),CODE( 2663),CODE( 2664),CODE( 2665)/
     *   Z033F,ZB000,Z0000,Z0000/
      DATA CODE( 2666),CODE( 2667),CODE( 2668),CODE( 2669)/
     *   Z43FC,Z0000,Z0000,Z0003/
      DATA CODE( 2670),CODE( 2671),CODE( 2672),CODE( 2673)/
     *   Z12D0,Z0000,Z4004,Z0000/
      DATA CODE( 2674),CODE( 2675),CODE( 2676),CODE( 2677)/
     *   Z03B8,Z0000,Z0700,Z0001/
      DATA CODE( 2678),CODE( 2679),CODE( 2680),CODE( 2681)/
     *   Z1314,Z0000,Z0000,Z0000/
      DATA CODE( 2682),CODE( 2683),CODE( 2684),CODE( 2685)/
     *   Z23F8,Z0000,Z0700,Z0003/
      DATA CODE( 2686),CODE( 2687),CODE( 2688),CODE( 2689)/
     *   Z1014,Z0012,Z0000,Z0000/
      DATA CODE( 2690),CODE( 2691),CODE( 2692),CODE( 2693)/
     *   Z12F4,Z0052,Z0820,Z0000/
      DATA CODE( 2694),CODE( 2695),CODE( 2696),CODE( 2697)/
     *   Z12F0,Z0000,Z0820,Z0000/
      DATA CODE( 2698),CODE( 2699),CODE( 2700),CODE( 2701)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2702),CODE( 2703),CODE( 2704),CODE( 2705)/
     *   Z033F,ZB000,Z0000,Z0000/
      DATA CODE( 2706),CODE( 2707),CODE( 2708),CODE( 2709)/
     *   Z43FC,Z0000,Z0000,Z0003/
      DATA CODE( 2710),CODE( 2711),CODE( 2712),CODE( 2713)/
     *   Z12D0,Z0000,Z4004,Z0000/
      DATA CODE( 2714),CODE( 2715),CODE( 2716),CODE( 2717)/
     *   Z03B4,Z0000,Z0700,Z0000/
      DATA CODE( 2718),CODE( 2719),CODE( 2720),CODE( 2721)/
     *   Z03F8,Z0000,Z0700,Z0000/
      DATA CODE( 2722),CODE( 2723),CODE( 2724),CODE( 2725)/
     *   Z4F70,Z0000,Z0000,Z0000/
      DATA CODE( 2726),CODE( 2727),CODE( 2728),CODE( 2729)/
     *   Z0270,Z0000,Z0000,Z0000/
      DATA CODE( 2730),CODE( 2731),CODE( 2732),CODE( 2733)/
     *   Z23F0,Z0000,Z0000,Z0003/
      DATA CODE( 2734),CODE( 2735),CODE( 2736),CODE( 2737)/
     *   Z1314,Z0000,Z0000,Z0000/
      DATA CODE( 2738),CODE( 2739),CODE( 2740),CODE( 2741)/
     *   Z02B8,Z0000,Z0000,Z0000/
      DATA CODE( 2742),CODE( 2743),CODE( 2744),CODE( 2745)/
     *   Z4374,Z005D,Z0000,Z0000/
      DATA CODE( 2746),CODE( 2747),CODE( 2748),CODE( 2749)/
     *   Z033F,ZB000,Z0000,Z0000/
      DATA CODE( 2750),CODE( 2751),CODE( 2752),CODE( 2753)/
     *   Z027C,Z0020,Z0000,Z0003/
      DATA CODE( 2754),CODE( 2755),CODE( 2756),CODE( 2757)/
     *   Z03B4,Z0000,Z0400,Z0002/
      DATA CODE( 2758),CODE( 2759),CODE( 2760),CODE( 2761)/
     *   Z03B8,Z0000,Z0400,ZFFFF/
      DATA CODE( 2762),CODE( 2763),CODE( 2764),CODE( 2765)/
     *   Z1314,Z0055,Z0000,Z0002/
      DATA CODE( 2766),CODE( 2767),CODE( 2768),CODE( 2769)/
     *   Z12D4,Z0000,Z4004,Z0000/
      DATA CODE( 2770),CODE( 2771),CODE( 2772),CODE( 2773)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE( 2774),CODE( 2775),CODE( 2776),CODE( 2777)/
     *   Z120C,Z0000,Z0700,Z0007/
      DATA CODE( 2778),CODE( 2779),CODE( 2780),CODE( 2781)/
     *   Z027C,Z0195,Z0000,Z0003/
      DATA CODE( 2782),CODE( 2783),CODE( 2784),CODE( 2785)/
     *   Z12D0,Z0000,Z4004,Z0000/
      DATA CODE( 2786),CODE( 2787),CODE( 2788),CODE( 2789)/
     *   Z0000,Z0000,Z0000,Z0000/
      DATA CODE( 2790),CODE( 2791),CODE( 2792),CODE( 2793)/
     *   Z120C,Z0000,Z0700,Z0003/
      DATA CODE( 2794),CODE( 2795),CODE( 2796),CODE( 2797)/
     *   Z43FC,Z01A9,Z0000,Z0003/
      DATA CODE( 2798),CODE( 2799),CODE( 2800),CODE( 2801)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2802),CODE( 2803),CODE( 2804),CODE( 2805)/
     *   Z1010,Z0016,Z0000,Z0000/
      DATA CODE( 2806),CODE( 2807),CODE( 2808),CODE( 2809)/
     *   Z03B0,Z0000,Z0700,Z0000/
      DATA CODE( 2810),CODE( 2811),CODE( 2812),CODE( 2813)/
     *   Z23F0,Z0000,Z0700,Z0003/
      DATA CODE( 2814),CODE( 2815),CODE( 2816),CODE( 2817)/
     *   Z1014,Z0012,Z0000,Z0000/
      DATA CODE( 2818),CODE( 2819),CODE( 2820),CODE( 2821)/
     *   Z12D4,Z0052,Z4004,Z0000/
      DATA CODE( 2822),CODE( 2823),CODE( 2824),CODE( 2825)/
     *   Z12D0,Z0000,Z4004,Z0000/
      DATA CODE( 2826),CODE( 2827),CODE( 2828),CODE( 2829)/
     *   Z027B,Z8200,Z0C00,Z0000/
      DATA CODE( 2830),CODE( 2831),CODE( 2832),CODE( 2833)/
     *   Z0380,Z0000,Z0700,Z0000/
      DATA CODE( 2834),CODE( 2835),CODE( 2836),CODE( 2837)/
     *   Z02B4,Z00E0,Z0000,Z0000/
      DATA CODE( 2838),CODE( 2839),CODE( 2840),CODE( 2841)/
     *   Z02C0,Z0000,Z0000,Z0000/
      DATA CODE( 2842),CODE( 2843),CODE( 2844),CODE( 2845)/
     *   Z0240,Z00E0,Z0000,Z0000/
      DATA CODE( 2846),CODE( 2847),CODE( 2848),CODE( 2849)/
     *   Z2040,Z00E0,Z0000,Z0000/
      DATA CODE( 2850),CODE( 2851),CODE( 2852),CODE( 2853)/
     *   Z3040,Z00E0,Z0000,Z0000/
      DATA CODE( 2854),CODE( 2855),CODE( 2856),CODE( 2857)/
     *   Z43FC,Z0000,Z1C00,Z8000/
      DATA CODE( 2858),CODE( 2859),CODE( 2860),CODE( 2861)/
     *   Z0040,Z0000,Z1404,Z021A/
      DATA CODE( 2862),CODE( 2863),CODE( 2864),CODE( 2865)/
     *   Z4000,Z0000,Z0000,Z0030/
      DATA CODE( 2866),CODE( 2867),CODE( 2868),CODE( 2869)/
     *   Z407F,Z8600,Z0400,Z0801/
      DATA CODE( 2870),CODE( 2871),CODE( 2872),CODE( 2873)/
     *   Z03B8,Z0000,Z0400,Z001B/
      DATA CODE( 2874),CODE( 2875),CODE( 2876),CODE( 2877)/
     *   Z43FC,Z0000,Z4C04,Z0000/
      DATA CODE( 2878),CODE( 2879),CODE( 2880),CODE( 2881)/
     *   Z43B9,ZDC00,Z0100,Z0000/
      DATA CODE( 2882),CODE( 2883),CODE( 2884),CODE( 2885)/
     *   Z4045,Z8000,Z0000,Z0000/
      DATA CODE( 2886),CODE( 2887),CODE( 2888),CODE( 2889)/
     *   Z023C,ZC394,Z0000,Z0000/
      DATA CODE( 2890),CODE( 2891),CODE( 2892),CODE( 2893)/
     *   Z0381,Z8000,Z0A00,Z0010/
      DATA CODE( 2894),CODE( 2895),CODE( 2896),CODE( 2897)/
     *   Z027C,ZC200,Z4C04,Z0000/
      DATA CODE( 2898),CODE( 2899),CODE( 2900),CODE( 2901)/
     *   Z0003,Z832E,Z0700,Z0000/
      DATA CODE( 2902),CODE( 2903),CODE( 2904),CODE( 2905)/
     *   Z12CC,Z0000,Z4004,Z000E/
      DATA CODE( 2906),CODE( 2907),CODE( 2908),CODE( 2909)/
     *   Z03BC,Z0000,Z0400,Z001A/
      DATA CODE( 2910),CODE( 2911),CODE( 2912),CODE( 2913)/
     *   Z43FE,ZBC00,Z0120,Z0000/
      DATA CODE( 2914),CODE( 2915),CODE( 2916),CODE( 2917)/
     *   Z0001,Z8000,Z0000,Z0000/
      DATA CODE( 2918),CODE( 2919),CODE( 2920),CODE( 2921)/
     *   Z0000,Z0000,Z2000,Z8000/
      DATA CODE( 2922),CODE( 2923),CODE( 2924),CODE( 2925)/
     *   Z037C,Z0118,Z0820,Z0000/
      DATA CODE( 2926),CODE( 2927),CODE( 2928),CODE( 2929)/
     *   Z43FE,ZBC00,Z4D25,Z0000/
      DATA CODE( 2930),CODE( 2931),CODE( 2932),CODE( 2933)/
     *   Z0003,Z8200,Z0740,Z0000/
      DATA CODE( 2934),CODE( 2935),CODE( 2936),CODE( 2937)/
     *   Z0381,Z8000,Z0000,Z0030/
      DATA CODE( 2938),CODE( 2939),CODE( 2940),CODE( 2941)/
     *   Z0000,Z0000,Z2000,Z8000/
      DATA CODE( 2942),CODE( 2943),CODE( 2944),CODE( 2945)/
     *   Z0003,Z8200,Z0740,Z0000/
      DATA CODE( 2946),CODE( 2947),CODE( 2948),CODE( 2949)/
     *   Z0380,Z0000,Z0A00,Z0000/
      DATA CODE( 2950),CODE( 2951),CODE( 2952),CODE( 2953)/
     *   Z037C,Z012A,Z0820,Z0000/
      DATA CODE( 2954),CODE( 2955),CODE( 2956),CODE( 2957)/
     *   Z0000,Z00E0,Z0000,Z0000/
      DATA CODE( 2958),CODE( 2959),CODE( 2960),CODE( 2961)/
     *   Z0000,Z0000,Z03FF,ZFFFF/
C-----------------------------------------------------------------------
      J1 = UV
      J2 = VIS
      J3 = WT
      J4 = L
      J5 = G
      J6 = CX
      J7 = CY
      J8 = N2
      J9 = M
      J10 = LROW
      J11 = CNT
      J12 = TY
      CALL APEX (CODE, 10, SLIST, 12)
 999  RETURN
      END
